@@ -28,6 +28,15 @@
 #include <set>
 #include <utility>
 
+// Managing compiler warnings for SDL headers
+#if defined( __GNUC__ )
+#pragma GCC diagnostic push
+
+#pragma GCC diagnostic ignored "-Wdouble-promotion"
+#pragma GCC diagnostic ignored "-Wold-style-cast"
+#pragma GCC diagnostic ignored "-Wswitch-default"
+#endif
+
 #include <SDL_error.h>
 #include <SDL_events.h>
 #include <SDL_hints.h>
@@ -39,14 +48,20 @@
 #include <SDL_surface.h>
 #include <SDL_video.h>
 
+// Managing compiler warnings for SDL headers
+#if defined( __GNUC__ )
+#pragma GCC diagnostic pop
+#endif
+
 #if defined( TARGET_PS_VITA )
 #include <vita2d.h>
 #endif
 
 #include "image_palette.h"
 #include "logging.h"
+#include "math_tools.h"
 #include "screen.h"
-#include "tools.h"
+#include "system.h"
 
 namespace
 {
@@ -621,15 +636,14 @@ namespace
             return filteredResolutions;
         }
 
-    protected:
-        RenderEngine()
-            : _window( nullptr )
-            , _surface( nullptr )
-            , _texBuffer( nullptr )
-            , _palettedTexturePointer( nullptr )
-        {
-            // Do nothing.
-        }
+    private:
+        SDL_Window * _window{ nullptr };
+        SDL_Surface * _surface{ nullptr };
+        vita2d_texture * _texBuffer{ nullptr };
+        uint8_t * _palettedTexturePointer{ nullptr };
+        fheroes2::Rect _destRect;
+
+        RenderEngine() = default;
 
         enum : int32_t
         {
@@ -739,13 +753,6 @@ namespace
             return true;
         }
 
-    private:
-        SDL_Window * _window;
-        SDL_Surface * _surface;
-        vita2d_texture * _texBuffer;
-        uint8_t * _palettedTexturePointer;
-        fheroes2::Rect _destRect;
-
         void _createPalette()
         {
             updatePalette( StandardPaletteIndexes() );
@@ -901,8 +908,11 @@ namespace
 
         void setTitle( const std::string & title ) override
         {
-            if ( _window != nullptr )
-                SDL_SetWindowTitle( _window, title.c_str() );
+            if ( _window == nullptr ) {
+                return;
+            }
+
+            SDL_SetWindowTitle( _window, System::encLocalToUTF8( title ).c_str() );
         }
 
         void setIcon( const fheroes2::Image & icon ) override
@@ -955,18 +965,23 @@ namespace
             }
         }
 
-    protected:
-        RenderEngine()
-            : _window( nullptr )
-            , _surface( nullptr )
-            , _renderer( nullptr )
-            , _texture( nullptr )
-            , _driverIndex( -1 )
-            , _prevWindowPos( SDL_WINDOWPOS_CENTERED, SDL_WINDOWPOS_CENTERED )
-            , _isVSyncEnabled( false )
-        {
-            // Do nothing.
-        }
+    private:
+        SDL_Window * _window{ nullptr };
+        SDL_Surface * _surface{ nullptr };
+        SDL_Renderer * _renderer{ nullptr };
+        SDL_Texture * _texture{ nullptr };
+        int _driverIndex{ -1 };
+
+        std::string _previousWindowTitle;
+        fheroes2::Point _prevWindowPos{ SDL_WINDOWPOS_CENTERED, SDL_WINDOWPOS_CENTERED };
+        fheroes2::Size _currentScreenResolution;
+        fheroes2::Rect _activeWindowROI;
+
+        fheroes2::Size _windowedSize;
+
+        bool _isVSyncEnabled{ false };
+
+        RenderEngine() = default;
 
         void clear() override
         {
@@ -985,7 +1000,7 @@ namespace
                 if ( !isFullScreen() ) {
                     SDL_GetWindowPosition( _window, &_prevWindowPos.x, &_prevWindowPos.y );
                 }
-                _previousWindowTitle = SDL_GetWindowTitle( _window );
+                _previousWindowTitle = System::encUTF8ToLocal( SDL_GetWindowTitle( _window ) );
 
                 SDL_DestroyWindow( _window );
                 _window = nullptr;
@@ -1069,7 +1084,7 @@ namespace
 #if defined( ANDROID )
             // Same as ActivityInfo.SCREEN_ORIENTATION_SENSOR_LANDSCAPE
             if ( SDL_SetHint( SDL_HINT_ORIENTATIONS, "LandscapeLeft LandscapeRight" ) == SDL_FALSE ) {
-                ERROR_LOG( "Failed to set a hint for screen orientation." )
+                ERROR_LOG( "Failed to set the " SDL_HINT_ORIENTATIONS " hint." )
             }
 #endif
 
@@ -1089,7 +1104,8 @@ namespace
 
             flags |= SDL_WINDOW_RESIZABLE;
 
-            _window = SDL_CreateWindow( _previousWindowTitle.data(), _prevWindowPos.x, _prevWindowPos.y, resolutionInfo.screenWidth, resolutionInfo.screenHeight, flags );
+            _window = SDL_CreateWindow( System::encLocalToUTF8( _previousWindowTitle ).c_str(), _prevWindowPos.x, _prevWindowPos.y, resolutionInfo.screenWidth,
+                                        resolutionInfo.screenHeight, flags );
             if ( _window == nullptr ) {
                 ERROR_LOG( "Failed to create an application window of " << resolutionInfo.screenWidth << " x " << resolutionInfo.screenHeight
                                                                         << " size. The error: " << SDL_GetError() )
@@ -1177,22 +1193,6 @@ namespace
             return ( _window != nullptr ) && ( ( SDL_GetWindowFlags( _window ) & SDL_WINDOW_MOUSE_FOCUS ) == SDL_WINDOW_MOUSE_FOCUS );
         }
 
-    private:
-        SDL_Window * _window;
-        SDL_Surface * _surface;
-        SDL_Renderer * _renderer;
-        SDL_Texture * _texture;
-        int _driverIndex;
-
-        std::string _previousWindowTitle;
-        fheroes2::Point _prevWindowPos;
-        fheroes2::Size _currentScreenResolution;
-        fheroes2::Rect _activeWindowROI;
-
-        fheroes2::Size _windowedSize;
-
-        bool _isVSyncEnabled;
-
         uint32_t renderFlags() const
         {
             if ( _isVSyncEnabled ) {
@@ -1292,13 +1292,13 @@ namespace
             }
 
             if ( SDL_SetHint( SDL_HINT_RENDER_SCALE_QUALITY, ( isNearestScaling() ? "nearest" : "linear" ) ) == SDL_FALSE ) {
-                ERROR_LOG( "Failed to set a linear scale hint for rendering." )
+                ERROR_LOG( "Failed to set the " SDL_HINT_RENDER_SCALE_QUALITY " hint." )
             }
 
             // Setting this hint prevents the window to regain focus after losing it in fullscreen mode.
             // It also fixes issues when SDL_UpdateTexture() calls fail because of refocusing.
             if ( SDL_SetHint( SDL_HINT_VIDEO_MINIMIZE_ON_FOCUS_LOSS, "0" ) == SDL_FALSE ) {
-                ERROR_LOG( "Failed to set a linear scale hint for rendering." )
+                ERROR_LOG( "Failed to set the " SDL_HINT_VIDEO_MINIMIZE_ON_FOCUS_LOSS " hint." )
             }
 
             returnCode = SDL_RenderSetLogicalSize( _renderer, width_, height_ );
