@@ -1,6 +1,6 @@
 /***************************************************************************
  *   fheroes2: https://github.com/ihhub/fheroes2                           *
- *   Copyright (C) 2019 - 2024                                             *
+ *   Copyright (C) 2019 - 2025                                             *
  *                                                                         *
  *   Free Heroes2 Engine: http://sourceforge.net/projects/fheroes2         *
  *   Copyright (C) 2009 by Andrey Afletdinov <fheroes2@gmail.com>          *
@@ -27,11 +27,10 @@
 #include <cassert>
 #include <cmath>
 #include <map>
-#include <memory>
 #include <numeric>
-#include <ostream>
 #include <random>
 #include <set>
+#include <sstream>
 #include <utility>
 
 #include "army_troop.h"
@@ -238,19 +237,16 @@ Troops::~Troops()
     } );
 }
 
-void Troops::Assign( const Troop * itbeg, const Troop * itend )
+void Troops::Assign( const Troop * troopsBegin, const Troop * troopsEnd )
 {
     Clean();
 
-    iterator it = begin();
+    for ( iterator iter = begin(); iter != end() && troopsBegin != troopsEnd; ++iter, ++troopsBegin ) {
+        assert( *iter != nullptr && troopsBegin != nullptr );
 
-    while ( it != end() && itbeg != itend ) {
-        if ( itbeg->isValid() ) {
-            ( *it )->Set( *itbeg );
+        if ( troopsBegin->isValid() ) {
+            ( *iter )->Set( *troopsBegin );
         }
-
-        ++it;
-        ++itbeg;
     }
 }
 
@@ -258,24 +254,25 @@ void Troops::Assign( const Troops & troops )
 {
     Clean();
 
-    iterator it1 = begin();
-    const_iterator it2 = troops.begin();
+    for ( auto [thisIter, troopsIter] = std::make_pair( begin(), troops.begin() ); thisIter != end() && troopsIter != troops.end(); ++thisIter, ++troopsIter ) {
+        assert( *thisIter != nullptr && *troopsIter != nullptr );
 
-    while ( it1 != end() && it2 != troops.end() ) {
-        if ( ( *it2 )->isValid() )
-            ( *it1 )->Set( **it2 );
-        ++it2;
-        ++it1;
+        if ( ( *troopsIter )->isValid() ) {
+            ( *thisIter )->Set( **troopsIter );
+        }
     }
 }
 
 void Troops::Insert( const Troops & troops )
 {
-    for ( const_iterator it = troops.begin(); it != troops.end(); ++it )
-        push_back( new Troop( **it ) );
+    for ( const Troop * troop : troops ) {
+        assert( troop != nullptr );
+
+        push_back( new Troop( *troop ) );
+    }
 }
 
-void Troops::PushBack( const Monster & mons, uint32_t count )
+void Troops::PushBack( const Monster & mons, const uint32_t count )
 {
     push_back( new Troop( mons, count ) );
 }
@@ -374,8 +371,7 @@ bool Troops::areAllTroopsUnique() const
             continue;
         }
 
-        auto [it, inserted] = monsterId.emplace( troop->GetID() );
-        if ( !inserted ) {
+        if ( auto [dummy, inserted] = monsterId.emplace( troop->GetID() ); !inserted ) {
             return false;
         }
     }
@@ -396,13 +392,20 @@ bool Troops::HasMonster( const Monster & mons ) const
 
 bool Troops::AllTroopsAreUndead() const
 {
+    bool isValidArmy = false;
+
     for ( const_iterator it = begin(); it != end(); ++it ) {
-        if ( ( *it )->isValid() && !( *it )->isUndead() ) {
-            return false;
+        if ( ( *it )->isValid() ) {
+            isValidArmy = true;
+
+            if ( !( *it )->isUndead() ) {
+                return false;
+            }
         }
     }
 
-    return true;
+    // If army has no units then the army cannot be marked as undead.
+    return isValidArmy;
 }
 
 bool Troops::CanJoinTroop( const Monster & mons ) const
@@ -506,6 +509,7 @@ void Troops::UpgradeTroops( const Castle & castle ) const
 {
     for ( Troop * troop : *this ) {
         assert( troop != nullptr );
+
         if ( !troop->isValid() ) {
             continue;
         }
@@ -514,7 +518,6 @@ void Troops::UpgradeTroops( const Castle & castle ) const
             continue;
         }
 
-        Kingdom & kingdom = castle.GetKingdom();
         if ( castle.GetRace() != troop->GetRace() ) {
             continue;
         }
@@ -523,18 +526,26 @@ void Troops::UpgradeTroops( const Castle & castle ) const
             continue;
         }
 
+        Kingdom & kingdom = castle.GetKingdom();
+
         const Funds payment = troop->GetTotalUpgradeCost();
-        if ( kingdom.AllowPayment( payment ) ) {
-            kingdom.OddFundsResource( payment );
-            troop->Upgrade();
+        if ( !kingdom.AllowPayment( payment ) ) {
+            continue;
         }
+
+        kingdom.OddFundsResource( payment );
+
+        troop->Upgrade();
     }
 }
 
-Troop * Troops::GetFirstValid()
+Troop * Troops::GetFirstValid() const
 {
-    iterator it = std::find_if( begin(), end(), []( const Troop * troop ) { return troop->isValid(); } );
-    return it == end() ? nullptr : *it;
+    if ( const const_iterator iter = std::find_if( begin(), end(), []( const Troop * troop ) { return troop->isValid(); } ); iter != end() ) {
+        return *iter;
+    }
+
+    return nullptr;
 }
 
 Troop * Troops::getBestMatchToCondition( const std::function<bool( const Troop *, const Troop * )> & condition ) const
@@ -991,7 +1002,7 @@ Army::Army( HeroBase * cmdr /* = nullptr */ )
     }
 }
 
-Army::Army( const Maps::Tiles & tile )
+Army::Army( const Maps::Tile & tile )
     : commander( nullptr )
     , _isSpreadCombatFormation( true )
     , color( Color::NONE )
@@ -1010,13 +1021,13 @@ const Troops & Army::getTroops() const
     return *this;
 }
 
-void Army::setFromTile( const Maps::Tiles & tile )
+void Army::setFromTile( const Maps::Tile & tile )
 {
     assert( commander == nullptr );
 
     Troops::Clean();
 
-    const bool isCaptureObject = MP2::isCaptureObject( tile.GetObject( false ) );
+    const bool isCaptureObject = MP2::isCaptureObject( tile.getMainObjectType( false ) );
     if ( isCaptureObject ) {
         color = getColorFromTile( tile );
     }
@@ -1024,7 +1035,7 @@ void Army::setFromTile( const Maps::Tiles & tile )
         color = Color::NONE;
     }
 
-    switch ( tile.GetObject( false ) ) {
+    switch ( tile.getMainObjectType( false ) ) {
     case MP2::OBJ_PYRAMID:
         at( 0 )->Set( Monster::VAMPIRE_LORD, 10 );
         at( 1 )->Set( Monster::ROYAL_MUMMY, 10 );
@@ -1201,18 +1212,13 @@ int Army::GetMoraleModificator( std::string * strs ) const
     // different race penalty
     std::set<int> races;
     bool hasUndead = false;
-    bool allUndead = true;
 
     for ( const Troop * troop : *this )
         if ( troop->isValid() ) {
             races.insert( troop->GetRace() );
             hasUndead = hasUndead || troop->isUndead();
-            allUndead = allUndead && troop->isUndead();
         }
 
-    if ( allUndead )
-        return Morale::NORMAL;
-
     int result = Morale::NORMAL;
 
     // check castle modificator
@@ -1270,7 +1276,7 @@ double Army::GetStrength() const
 {
     double result = 0;
 
-    const uint32_t heroArchery = ( commander != nullptr ) ? commander->GetSecondaryValues( Skill::Secondary::ARCHERY ) : 0;
+    const uint32_t heroArchery = ( commander != nullptr ) ? commander->GetSecondarySkillValue( Skill::Secondary::ARCHERY ) : 0;
 
     const int bonusAttack = ( commander ? commander->GetAttack() : 0 );
     const int bonusDefense = ( commander ? commander->GetDefense() : 0 );
@@ -1377,16 +1383,26 @@ std::string Army::String() const
 {
     std::ostringstream os;
 
-    os << "color(" << Color::String( commander ? commander->GetColor() : color ) << "), ";
+    os << "color(" << Color::String( GetColor() ) << "), strength(" << GetStrength() << "), ";
 
-    if ( GetCommander() )
+    if ( const HeroBase * cmdr = GetCommander(); cmdr != nullptr ) {
         os << "commander(" << GetCommander()->GetName() << ")";
+    }
+    else {
+        os << "commander(None)";
+    }
 
     os << ": ";
 
-    for ( const_iterator it = begin(); it != end(); ++it )
-        if ( ( *it )->isValid() )
-            os << std::dec << ( *it )->GetCount() << " " << ( *it )->GetName() << ", ";
+    for ( const Troop * troop : *this ) {
+        assert( troop != nullptr );
+
+        if ( !troop->isValid() ) {
+            continue;
+        }
+
+        os << std::dec << troop->GetCount() << " " << troop->GetName() << ", ";
+    }
 
     return os.str();
 }
@@ -1493,6 +1509,20 @@ void Army::MoveTroops( Army & from, const int monsterIdToKeep )
     moveTroops( false );
 }
 
+void Army::SwapTroops( Army & from )
+{
+    assert( this != &from );
+
+    // Heroes need to have at least one occupied slot in their army, so both armies should have at least one
+    // occupied slot, even if the exchange of armies takes place between a castle garrison and a hero.
+    assert( from.isValid() && isValid() );
+
+    const Troops temp = this->getTroops();
+
+    Assign( from );
+    from.Assign( temp );
+}
+
 uint32_t Army::ActionToSirens() const
 {
     uint32_t experience = 0;
@@ -1586,8 +1616,14 @@ void Army::drawMultipleMonsterLines( const Troops & troops, int32_t posX, int32_
     }
 }
 
-NeutralMonsterJoiningCondition Army::GetJoinSolution( const Heroes & hero, const Maps::Tiles & tile, const Troop & troop )
+NeutralMonsterJoiningCondition Army::GetJoinSolution( const Heroes & hero, const Maps::Tile & tile, const Troop & troop )
 {
+    if ( !troop.isValid() ) {
+        assert( 0 );
+
+        return { NeutralMonsterJoiningCondition::Reason::None, 0, nullptr, nullptr };
+    }
+
     // Check for creature alliance/bane campaign awards, campaign only and of course, for human players
     // creature alliance -> if we have an alliance with the appropriate creature (inc. players) they will join for free
     // creature curse/bane -> same as above but all of them will flee even if you have just 1 peasant
@@ -1598,8 +1634,9 @@ NeutralMonsterJoiningCondition Army::GetJoinSolution( const Heroes & hero, const
             const bool isAlliance = campaignAwards[i]._type == Campaign::CampaignAwardData::TYPE_CREATURE_ALLIANCE;
             const bool isCurse = campaignAwards[i]._type == Campaign::CampaignAwardData::TYPE_CREATURE_CURSE;
 
-            if ( !isAlliance && !isCurse )
+            if ( !isAlliance && !isCurse ) {
                 continue;
+            }
 
             Monster monster( campaignAwards[i]._subType );
             while ( true ) {
@@ -1615,38 +1652,48 @@ NeutralMonsterJoiningCondition Army::GetJoinSolution( const Heroes & hero, const
                     }
                 }
 
-                // try to cycle through the creature's upgrades
-                if ( !monster.isAllowUpgrade() )
+                // Also check the upgraded variant(s) of the same monster
+                if ( !monster.isAllowUpgrade() ) {
                     break;
+                }
 
                 monster = monster.GetUpgrade();
             }
         }
     }
 
-    if ( hero.GetBagArtifacts().isArtifactCursePresent( fheroes2::ArtifactCurseType::NO_JOINING_ARMIES ) ) {
-        return { NeutralMonsterJoiningCondition::Reason::None, 0, nullptr, nullptr };
-    }
-
-    if ( Maps::isMonsterOnTileJoinConditionSkip( tile ) || !troop.isValid() ) {
-        return { NeutralMonsterJoiningCondition::Reason::None, 0, nullptr, nullptr };
-    }
-
     // Neutral monsters don't care about hero's stats. Ignoring hero's stats makes hero's army strength be smaller in eyes of neutrals and they won't join so often.
     const double armyStrengthRatio = Troops( hero.GetArmy().getTroops() ).GetStrength() / troop.GetStrength();
 
-    // The ability to accept monsters (a free slot or a stack of monsters of the same type) is a
-    // mandatory condition for their joining in accordance with the mechanics of the original game
-    if ( armyStrengthRatio > 2 && hero.GetArmy().CanJoinTroop( troop ) ) {
+    const bool canJoin = [&hero, &tile, &troop, armyStrengthRatio]() {
+        if ( armyStrengthRatio <= 2 ) {
+            return false;
+        }
+
+        // The ability to accept monsters (a free slot or a stack of monsters of the same type) is a mandatory condition for their joining in accordance with the
+        // mechanics of the original game
+        if ( !hero.GetArmy().CanJoinTroop( troop ) ) {
+            return false;
+        }
+
+        if ( hero.GetBagArtifacts().isArtifactCursePresent( fheroes2::ArtifactCurseType::NO_JOINING_ARMIES ) ) {
+            return false;
+        }
+
+        return !Maps::isMonsterOnTileJoinConditionSkip( tile );
+    }();
+
+    if ( canJoin ) {
         if ( Maps::isMonsterOnTileJoinConditionFree( tile ) ) {
             return { NeutralMonsterJoiningCondition::Reason::Free, troop.GetCount(), nullptr, nullptr };
         }
 
         if ( hero.HasSecondarySkill( Skill::Secondary::DIPLOMACY ) ) {
-            const uint32_t amountToJoin = Monster::GetCountFromHitPoints( troop, troop.GetHitPoints() * hero.GetSecondaryValues( Skill::Secondary::DIPLOMACY ) / 100 );
+            const uint32_t amountToJoin
+                = Monster::GetCountFromHitPoints( troop, troop.GetHitPoints() * hero.GetSecondarySkillValue( Skill::Secondary::DIPLOMACY ) / 100 );
 
-            // The ability to hire the entire stack of monsters is a mandatory condition for their joining
-            // due to hero's Diplomacy skill in accordance with the mechanics of the original game
+            // The ability to immediately hire the entire stack of monsters is a mandatory condition for their joining due to hero's Diplomacy skill in accordance with
+            // the mechanics of the original game
             if ( amountToJoin > 0 && hero.GetKingdom().AllowPayment( Funds( Resource::GOLD, troop.GetTotalCost().gold ) ) ) {
                 return { NeutralMonsterJoiningCondition::Reason::ForMoney, amountToJoin, nullptr, nullptr };
             }
@@ -1654,7 +1701,7 @@ NeutralMonsterJoiningCondition Army::GetJoinSolution( const Heroes & hero, const
     }
 
     if ( armyStrengthRatio > 5 && !hero.isControlAI() ) {
-        // ... surely flee before us
+        // ... will surely flee before us
         return { NeutralMonsterJoiningCondition::Reason::RunAway, 0, nullptr, nullptr };
     }
 
@@ -1905,36 +1952,49 @@ void Army::ArrangeForBattle( const Monster & monster, const uint32_t monstersCou
     }
 }
 
-StreamBase & operator<<( StreamBase & msg, const Army & army )
+OStreamBase & operator<<( OStreamBase & stream, const Army & army )
 {
-    msg << static_cast<uint32_t>( army.size() );
+    stream.put32( static_cast<uint32_t>( army.size() ) );
 
-    // Army: fixed size
-    for ( Army::const_iterator it = army.begin(); it != army.end(); ++it )
-        msg << **it;
+    std::for_each( army.begin(), army.end(), [&stream]( const Troop * troop ) {
+        assert( troop != nullptr );
 
-    return msg << army._isSpreadCombatFormation << army.color;
+        stream << *troop;
+    } );
+
+    return stream << army._isSpreadCombatFormation << army.color;
 }
 
-StreamBase & operator>>( StreamBase & msg, Army & army )
+IStreamBase & operator>>( IStreamBase & stream, Army & army )
 {
-    uint32_t armysz;
-    msg >> armysz;
+    if ( const uint32_t size = stream.get32(); army.size() != size ) {
+        // Most likely the save file is corrupted.
+        stream.setFail();
 
-    for ( Army::iterator it = army.begin(); it != army.end(); ++it )
-        msg >> **it;
+        std::for_each( army.begin(), army.end(), []( Troop * troop ) {
+            assert( troop != nullptr );
 
-    msg >> army._isSpreadCombatFormation >> army.color;
+            troop->Reset();
+        } );
+    }
+    else {
+        std::for_each( army.begin(), army.end(), [&stream]( Troop * troop ) {
+            assert( troop != nullptr );
 
-    // set army
-    for ( Army::iterator it = army.begin(); it != army.end(); ++it ) {
-        ArmyTroop * troop = static_cast<ArmyTroop *>( *it );
-        if ( troop )
-            troop->SetArmy( army );
+            stream >> *troop;
+        } );
     }
 
-    // set later from owner (castle, heroes)
+    stream >> army._isSpreadCombatFormation >> army.color;
+
+    assert( std::all_of( army.begin(), army.end(), [&army]( const Troop * troop ) {
+        const ArmyTroop * armyTroop = dynamic_cast<const ArmyTroop *>( troop );
+
+        return armyTroop != nullptr && armyTroop->GetArmy() == &army;
+    } ) );
+
+    // Will be set later by the owner (castle or hero)
     army.commander = nullptr;
 
-    return msg;
+    return stream;
 }
