@@ -622,7 +622,7 @@ namespace
             const ListFiles files = Settings::FindFiles( "maps", "", false );
 
             for ( const std::string & file : files ) {
-                result.try_emplace( StringLower( System::GetBasename( file ) ), file );
+                result.try_emplace( StringLower( System::GetFileName( file ) ), file );
             }
 
             return result;
@@ -642,14 +642,14 @@ namespace
 
 namespace Campaign
 {
-    StreamBase & operator<<( StreamBase & msg, const ScenarioInfoId & data )
+    OStreamBase & operator<<( OStreamBase & stream, const ScenarioInfoId & data )
     {
-        return msg << data.campaignId << data.scenarioId;
+        return stream << data.campaignId << data.scenarioId;
     }
 
-    StreamBase & operator>>( StreamBase & msg, ScenarioInfoId & data )
+    IStreamBase & operator>>( IStreamBase & stream, ScenarioInfoId & data )
     {
-        return msg >> data.campaignId >> data.scenarioId;
+        return stream >> data.campaignId >> data.scenarioId;
     }
 
     ScenarioBonusData::ScenarioBonusData()
