@@ -34,7 +34,7 @@
 
 namespace Maps
 {
-    class Tiles;
+    class Tile;
 
     enum class ObjectGroup : uint8_t;
 }
@@ -66,7 +66,11 @@ namespace Interface
 
         void mouseCursorAreaClickLeft( const int32_t tileIndex ) override;
         void mouseCursorAreaPressRight( const int32_t tileIndex ) const override;
-        void mouseCursorAreaLongPressLeft( const int32_t /*Unused*/ ) override;
+
+        void mouseCursorAreaLongPressLeft( const int32_t /*Unused*/ ) override
+        {
+            // Do nothing.
+        }
 
         void undoAction()
         {
@@ -140,17 +144,24 @@ namespace Interface
             // Do nothing.
         }
 
-        bool _setObjectOnTile( Maps::Tiles & tile, const Maps::ObjectGroup groupType, const int32_t objectIndex );
+        bool _setObjectOnTile( Maps::Tile & tile, const Maps::ObjectGroup groupType, const int32_t objectIndex );
 
-        bool _setObjectOnTileAsAction( Maps::Tiles & tile, const Maps::ObjectGroup groupType, const int32_t objectIndex );
+        bool _setObjectOnTileAsAction( Maps::Tile & tile, const Maps::ObjectGroup groupType, const int32_t objectIndex );
 
-        void _handleObjectMouseLeftClick( Maps::Tiles & tile );
+        void _handleObjectMouseLeftClick( Maps::Tile & tile );
 
         void _validateObjectsOnTerrainUpdate();
 
+        // Returns true if an existing object was moved.
+        bool _moveExistingObject( const int32_t tileIndex, const Maps::ObjectGroup groupType, int32_t objectIndex );
+
+        void _updateObjectMetadata( const Maps::Map_Format::TileObjectInfo & object, const uint32_t newObjectUID );
+
+        void _updateObjectUID( const uint32_t oldObjectUID, const uint32_t newObjectUID );
+
         EditorPanel _editorPanel;
 
-        int32_t _selectedTile{ -1 };
+        int32_t _areaSelectionStartTileId{ -1 };
         int32_t _tileUnderCursor{ -1 };
 
         std::function<void( const int32_t )> _cursorUpdater;
