@@ -1,6 +1,6 @@
 /***************************************************************************
  *   fheroes2: https://github.com/ihhub/fheroes2                           *
- *   Copyright (C) 2019 - 2024                                             *
+ *   Copyright (C) 2019 - 2025                                             *
  *                                                                         *
  *   Free Heroes2 Engine: http://sourceforge.net/projects/fheroes2         *
  *   Copyright (C) 2010 by Andrey Afletdinov <fheroes2@gmail.com>          *
@@ -27,11 +27,11 @@
 #include <array>
 #include <cassert>
 #include <cstring>
-#include <fstream>
 #include <functional>
 #include <map>
 #include <set>
 #include <sstream>
+#include <string_view>
 #include <type_traits>
 #include <utility>
 
@@ -43,6 +43,7 @@
 #include "localevent.h"
 #include "logging.h"
 #include "players.h"
+#include "serialize.h"
 #include "settings.h"
 #include "system.h"
 #include "tinyconfig.h"
@@ -107,6 +108,11 @@ namespace
         hotKeyEventInfo[hotKeyEventToInt( Game::HotKeyEvent::GLOBAL_TOGGLE_TEXT_SUPPORT_MODE )]
             = { Game::HotKeyCategory::GLOBAL, gettext_noop( "hotkey|toggle text support mode" ), fheroes2::Key::KEY_F10 };
 
+#if defined( WITH_DEBUG )
+        hotKeyEventInfo[hotKeyEventToInt( Game::HotKeyEvent::DEBUG_TOGGLE_DEVELOPER_MODE )]
+            = { Game::HotKeyCategory::GLOBAL, gettext_noop( "hotkey|toggle developer mode" ), fheroes2::Key::KEY_BACKQUOTE };
+#endif
+
         hotKeyEventInfo[hotKeyEventToInt( Game::HotKeyEvent::MAIN_MENU_NEW_GAME )]
             = { Game::HotKeyCategory::MAIN_MENU, gettext_noop( "hotkey|new game" ), fheroes2::Key::KEY_N };
         hotKeyEventInfo[hotKeyEventToInt( Game::HotKeyEvent::MAIN_MENU_LOAD_GAME )]
@@ -269,10 +275,10 @@ namespace
             = { Game::HotKeyCategory::BATTLE, gettext_noop( "hotkey|retreat from battle" ), fheroes2::Key::KEY_R };
         hotKeyEventInfo[hotKeyEventToInt( Game::HotKeyEvent::BATTLE_SURRENDER )]
             = { Game::HotKeyCategory::BATTLE, gettext_noop( "hotkey|surrender during battle" ), fheroes2::Key::KEY_S };
-        hotKeyEventInfo[hotKeyEventToInt( Game::HotKeyEvent::BATTLE_AUTO_SWITCH )]
-            = { Game::HotKeyCategory::BATTLE, gettext_noop( "hotkey|toggle battle auto mode" ), fheroes2::Key::KEY_A };
-        hotKeyEventInfo[hotKeyEventToInt( Game::HotKeyEvent::BATTLE_AUTO_FINISH )]
-            = { Game::HotKeyCategory::BATTLE, gettext_noop( "hotkey|finish the battle in auto mode" ), fheroes2::Key::KEY_Q };
+        hotKeyEventInfo[hotKeyEventToInt( Game::HotKeyEvent::BATTLE_TOGGLE_AUTO_COMBAT )]
+            = { Game::HotKeyCategory::BATTLE, gettext_noop( "hotkey|toggle auto combat mode" ), fheroes2::Key::KEY_A };
+        hotKeyEventInfo[hotKeyEventToInt( Game::HotKeyEvent::BATTLE_QUICK_COMBAT )]
+            = { Game::HotKeyCategory::BATTLE, gettext_noop( "hotkey|quick combat" ), fheroes2::Key::KEY_Q };
         hotKeyEventInfo[hotKeyEventToInt( Game::HotKeyEvent::BATTLE_OPTIONS )]
             = { Game::HotKeyCategory::BATTLE, gettext_noop( "hotkey|battle options" ), fheroes2::Key::KEY_O };
         hotKeyEventInfo[hotKeyEventToInt( Game::HotKeyEvent::BATTLE_SKIP )]
@@ -316,6 +322,8 @@ namespace
             = { Game::HotKeyCategory::ARMY, gettext_noop( "hotkey|upgrade troop" ), fheroes2::Key::KEY_U };
         hotKeyEventInfo[hotKeyEventToInt( Game::HotKeyEvent::ARMY_DISMISS )]
             = { Game::HotKeyCategory::ARMY, gettext_noop( "hotkey|dismiss hero or troop" ), fheroes2::Key::KEY_D };
+        hotKeyEventInfo[hotKeyEventToInt( Game::HotKeyEvent::ARMY_SWAP )]
+            = { Game::HotKeyCategory::ARMY, gettext_noop( "hotkey|exchange all troops" ), fheroes2::Key::KEY_S };
     }
 
     std::string getHotKeyFileContent()
@@ -430,7 +438,22 @@ void Game::HotKeysLoad( const std::string & filename )
                 const char * eventName = _( hotKeyEventInfo[eventId].name );
                 std::string value = config.StrParams( eventName );
                 if ( value.empty() ) {
-                    continue;
+                    // TODO: remove this temporary workaround
+                    if ( eventName == std::string_view( "toggle auto combat mode" ) ) {
+                        value = config.StrParams( "toggle battle auto mode" );
+                        if ( value.empty() ) {
+                            continue;
+                        }
+                    }
+                    else if ( eventName == std::string_view( "quick combat" ) ) {
+                        value = config.StrParams( "finish the battle in auto mode" );
+                        if ( value.empty() ) {
+                            continue;
+                        }
+                    }
+                    else {
+                        continue;
+                    }
                 }
 
                 value = StringUpper( value );
@@ -450,17 +473,17 @@ void Game::HotKeysLoad( const std::string & filename )
 
 void Game::HotKeySave()
 {
-    // Save the latest information into the file.
     const std::string filename = System::concatPath( System::GetConfigDirectory( "fheroes2" ), "fheroes2.key" );
 
-    std::fstream file( filename.data(), std::fstream::out | std::fstream::trunc );
-    if ( !file ) {
-        ERROR_LOG( "Unable to open hotkey settings file " << filename )
+    StreamFile fileStream;
+    if ( !fileStream.open( filename, "w" ) ) {
+        ERROR_LOG( "Unable to open the hotkey settings file " << filename )
         return;
     }
 
-    const std::string & data = getHotKeyFileContent();
-    file.write( data.data(), data.size() );
+    const std::string data = getHotKeyFileContent();
+
+    fileStream.putRaw( data.data(), data.size() );
 }
 
 void Game::globalKeyDownEvent( const fheroes2::Key key, const int32_t modifier )
@@ -480,6 +503,9 @@ void Game::globalKeyDownEvent( const fheroes2::Key key, const int32_t modifier )
         conf.Save( Settings::configFileName );
     }
 #if defined( WITH_DEBUG )
+    else if ( key == hotKeyEventInfo[hotKeyEventToInt( HotKeyEvent::DEBUG_TOGGLE_DEVELOPER_MODE )].key ) {
+        Logging::setDebugLevel( DBG_DEVEL ^ Logging::getDebugLevel() );
+    }
     else if ( key == hotKeyEventInfo[hotKeyEventToInt( HotKeyEvent::WORLD_TRANSFER_CONTROL_TO_AI )].key ) {
         static bool recursiveCall = false;
 
