@@ -1,6 +1,6 @@
 /***************************************************************************
  *   fheroes2: https://github.com/ihhub/fheroes2                           *
- *   Copyright (C) 2019 - 2024                                             *
+ *   Copyright (C) 2019 - 2025                                             *
  *                                                                         *
  *   Free Heroes2 Engine: http://sourceforge.net/projects/fheroes2         *
  *   Copyright (C) 2009 by Andrey Afletdinov <fheroes2@gmail.com>          *
@@ -178,7 +178,18 @@ fheroes2::GameMode Interface::ButtonsPanel::queueEventProcessing()
     else if ( _buttonHeroMovement.isEnabled() && le.MouseClickLeft( _heroMovementRect ) ) {
         res = _interface.EventHeroMovement();
     }
-    else if ( _buttonHeroMovement.isEnabled() && le.MouseLongPressLeft( _heroMovementRect ) ) {
+    // The 'MouseLongPressLeft()' for '_heroMovementRect' should be called only if current hero
+    // has valid path not to trigger the '_mouseButtonLongPressDelay'.
+    // The inlined 'isMouseLeftButtonPressedInArea()' method is called not to call 'isValidForMovement()'
+    // method every time this button is enabled.
+    else if ( _buttonHeroMovement.isEnabled() && le.isMouseLeftButtonPressedInArea( _heroMovementRect ) &&
+              []() {
+                  const Heroes * currentHero = GetFocusHeroes();
+                  assert( currentHero != nullptr );
+
+                  return currentHero->GetPath().isValidForMovement();
+              }()
+              && le.MouseLongPressLeft( _heroMovementRect ) ) {
         _interface.EventResetHeroPath();
     }
     else if ( le.MouseClickLeft( _kingdomRect ) ) {
@@ -236,15 +247,23 @@ void Interface::ButtonsPanel::_setButtonStatus()
     Heroes * currentHero = GetFocusHeroes();
 
     if ( currentHero && currentHero->GetPath().isValidForMovement() && currentHero->MayStillMove( false, true ) ) {
+        // Move button: to move the hero by the specified path.
         _buttonHeroMovement.setICNIndexes( 2, 3 );
         _buttonHeroMovement.enable();
     }
     else if ( currentHero && MP2::isInGameActionObject( currentHero->getObjectTypeUnderHero(), currentHero->isShipMaster() ) ) {
+        // Action button: to visit the action object hero is standing on.
         _buttonHeroMovement.setICNIndexes( 16, 17 );
         _buttonHeroMovement.enable();
     }
+    else if ( currentHero && currentHero->GetPath().isValidForMovement() ) {
+        // Inactive button: to show the hero's path can be reset.
+        _buttonHeroMovement.setICNIndexes( 18, 19 );
+        _buttonHeroMovement.enable();
+    }
     else {
-        _buttonHeroMovement.setICNIndexes( 2, 3 );
+        // Inactive button (disabled): to show that the hero as no path specified.
+        _buttonHeroMovement.setICNIndexes( 18, 19 );
         _buttonHeroMovement.disable();
     }
 
