@@ -1,6 +1,6 @@
 /***************************************************************************
  *   fheroes2: https://github.com/ihhub/fheroes2                           *
- *   Copyright (C) 2019 - 2024                                             *
+ *   Copyright (C) 2019 - 2025                                             *
  *                                                                         *
  *   Free Heroes2 Engine: http://sourceforge.net/projects/fheroes2         *
  *   Copyright (C) 2010 by Andrey Afletdinov <fheroes2@gmail.com>          *
@@ -21,8 +21,7 @@
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
  ***************************************************************************/
 
-#ifndef H2INTERFACE_LIST_H
-#define H2INTERFACE_LIST_H
+#pragma once
 
 #include <algorithm>
 
@@ -57,11 +56,11 @@ namespace Interface
     public:
         explicit ListBox( const fheroes2::Point & pt = fheroes2::Point() )
             : ptRedraw( pt )
-            , _timedButtonPgUp( [this]() { return buttonPgUp.isPressed(); } )
-            , _timedButtonPgDn( [this]() { return buttonPgDn.isPressed(); } )
+            , _timedButtonPgUp( [this]() { return _buttonPgUp.isPressed(); } )
+            , _timedButtonPgDn( [this]() { return _buttonPgDn.isPressed(); } )
         {
-            buttonPgUp.subscribe( &_timedButtonPgUp );
-            buttonPgDn.subscribe( &_timedButtonPgDn );
+            _buttonPgUp.subscribe( &_timedButtonPgUp );
+            _buttonPgDn.subscribe( &_timedButtonPgDn );
         }
         ~ListBox() override = default;
 
@@ -102,14 +101,14 @@ namespace Interface
 
         void SetScrollButtonUp( int icn, uint32_t index1, uint32_t index2, const fheroes2::Point & pos )
         {
-            buttonPgUp.setICNInfo( icn, index1, index2 );
-            buttonPgUp.setPosition( pos.x, pos.y );
+            _buttonPgUp.setICNInfo( icn, index1, index2 );
+            _buttonPgUp.setPosition( pos.x, pos.y );
         }
 
         void SetScrollButtonDn( int icn, uint32_t index1, uint32_t index2, const fheroes2::Point & pos )
         {
-            buttonPgDn.setICNInfo( icn, index1, index2 );
-            buttonPgDn.setPosition( pos.x, pos.y );
+            _buttonPgDn.setICNInfo( icn, index1, index2 );
+            _buttonPgDn.setPosition( pos.x, pos.y );
         }
 
         void setScrollBarArea( const fheroes2::Rect & area )
@@ -176,8 +175,8 @@ namespace Interface
 
             RedrawBackground( ptRedraw );
 
-            buttonPgUp.draw();
-            buttonPgDn.draw();
+            _buttonPgUp.draw();
+            _buttonPgDn.draw();
             _scrollbar.redraw();
 
             Verify(); // reset values if they are wrong
@@ -334,14 +333,14 @@ namespace Interface
         {
             LocalEvent & le = LocalEvent::Get();
 
-            le.MousePressLeft( buttonPgUp.area() ) ? buttonPgUp.drawOnPress() : buttonPgUp.drawOnRelease();
-            le.MousePressLeft( buttonPgDn.area() ) ? buttonPgDn.drawOnPress() : buttonPgDn.drawOnRelease();
+            _buttonPgUp.drawOnState( le.isMouseLeftButtonPressedInArea( _buttonPgUp.area() ) );
+            _buttonPgDn.drawOnState( le.isMouseLeftButtonPressedInArea( _buttonPgDn.area() ) );
 
             if ( !IsValid() ) {
                 return false;
             }
 
-            if ( useHotkeys && le.KeyPress( fheroes2::Key::KEY_PAGE_UP ) && ( _topId > 0 ) ) {
+            if ( useHotkeys && le.isKeyPressed( fheroes2::Key::KEY_PAGE_UP ) && ( _topId > 0 ) ) {
                 needRedraw = true;
 
                 if ( _topId > maxItems ) {
@@ -356,7 +355,7 @@ namespace Interface
 
                 return true;
             }
-            if ( useHotkeys && le.KeyPress( fheroes2::Key::KEY_PAGE_DOWN ) && ( _topId + maxItems < _size() ) ) {
+            if ( useHotkeys && le.isKeyPressed( fheroes2::Key::KEY_PAGE_DOWN ) && ( _topId + maxItems < _size() ) ) {
                 needRedraw = true;
 
                 _topId += maxItems;
@@ -369,7 +368,7 @@ namespace Interface
 
                 return true;
             }
-            if ( useHotkeys && le.KeyPress( fheroes2::Key::KEY_UP ) && ( _currentId > 0 ) ) {
+            if ( useHotkeys && le.isKeyPressed( fheroes2::Key::KEY_UP ) && ( _currentId > 0 ) ) {
                 needRedraw = true;
 
                 --_currentId;
@@ -378,7 +377,7 @@ namespace Interface
 
                 return true;
             }
-            if ( useHotkeys && le.KeyPress( fheroes2::Key::KEY_DOWN ) && ( _currentId + 1 < _size() ) ) {
+            if ( useHotkeys && le.isKeyPressed( fheroes2::Key::KEY_DOWN ) && ( _currentId + 1 < _size() ) ) {
                 needRedraw = true;
 
                 ++_currentId;
@@ -387,7 +386,7 @@ namespace Interface
 
                 return true;
             }
-            if ( useHotkeys && le.KeyPress( fheroes2::Key::KEY_HOME ) && ( _topId > 0 ) ) {
+            if ( useHotkeys && le.isKeyPressed( fheroes2::Key::KEY_HOME ) && ( _topId > 0 ) ) {
                 needRedraw = true;
 
                 _topId = 0;
@@ -397,7 +396,7 @@ namespace Interface
 
                 return true;
             }
-            if ( useHotkeys && le.KeyPress( fheroes2::Key::KEY_END ) && ( _topId + maxItems < _size() ) ) {
+            if ( useHotkeys && le.isKeyPressed( fheroes2::Key::KEY_END ) && ( _topId + maxItems < _size() ) ) {
                 needRedraw = true;
 
                 _topId = _size() - maxItems;
@@ -407,7 +406,7 @@ namespace Interface
 
                 return true;
             }
-            if ( ( le.MouseClickLeft( buttonPgUp.area() ) || le.MouseWheelUp( rtAreaItems ) || le.MouseWheelUp( _scrollbar.getArea() )
+            if ( ( le.MouseClickLeft( _buttonPgUp.area() ) || le.isMouseWheelUpInArea( rtAreaItems ) || le.isMouseWheelUpInArea( _scrollbar.getArea() )
                    || _timedButtonPgUp.isDelayPassed() )
                  && ( _topId > 0 ) ) {
                 needRedraw = true;
@@ -417,7 +416,7 @@ namespace Interface
 
                 return true;
             }
-            if ( ( le.MouseClickLeft( buttonPgDn.area() ) || le.MouseWheelDn( rtAreaItems ) || le.MouseWheelDn( _scrollbar.getArea() )
+            if ( ( le.MouseClickLeft( _buttonPgDn.area() ) || le.isMouseWheelDownInArea( rtAreaItems ) || le.isMouseWheelDownInArea( _scrollbar.getArea() )
                    || _timedButtonPgDn.isDelayPassed() )
                  && ( _topId + maxItems < _size() ) ) {
                 needRedraw = true;
@@ -427,19 +426,19 @@ namespace Interface
 
                 return true;
             }
-            if ( le.MousePressLeft( _scrollbar.getArea() ) || le.MousePressLeft( rtAreaItems ) ) {
-                const fheroes2::Point mousePosition = le.GetMouseCursor();
+            if ( le.isMouseLeftButtonPressedInArea( _scrollbar.getArea() ) || le.isMouseLeftButtonPressedInArea( rtAreaItems ) ) {
+                const fheroes2::Point mousePosition = le.getMouseCursorPos();
 
                 const int32_t prevScrollbarX = _scrollbar.x();
                 const int32_t prevScrollbarY = _scrollbar.y();
 
                 UpdateScrollbarRange();
 
-                if ( le.MousePressLeft( _scrollbar.getArea() ) && ( _size() > maxItems ) ) {
+                if ( le.isMouseLeftButtonPressedInArea( _scrollbar.getArea() ) && ( _size() > maxItems ) ) {
                     _scrollbar.moveToPos( mousePosition );
                 }
 
-                if ( le.MousePressLeft( rtAreaItems ) ) {
+                if ( le.isMouseLeftButtonPressedInArea( rtAreaItems ) ) {
                     if ( !le.isDragInProgress() ) {
                         // Remember where has the drag started.
                         _dragStartPos = mousePosition;
@@ -484,7 +483,7 @@ namespace Interface
                 }
             }
 
-            const fheroes2::Point & mousePos = le.GetMouseCursor();
+            const fheroes2::Point & mousePos = le.getMouseCursorPos();
             if ( rtAreaItems & mousePos ) { // within our rectangle
                 needRedraw = true;
 
@@ -494,8 +493,9 @@ namespace Interface
                     Item & item = ( *content )[static_cast<size_t>( id )]; // id is always >= 0
                     const int32_t offsetY = ( id - _topId ) * rtAreaItems.height / maxItems;
 
-                    if ( ActionListCursor( item, mousePos ) )
+                    if ( ActionListCursor( item, mousePos ) ) {
                         return true;
+                    }
 
                     if ( !_lockClick && le.MouseClickLeft( rtAreaItems ) ) {
                         // This is a legitimate click and not a mouse-up on a finished drag.
@@ -509,7 +509,7 @@ namespace Interface
                         return true;
                     }
 
-                    if ( le.MousePressRight( rtAreaItems ) ) {
+                    if ( le.isMouseRightButtonPressedInArea( rtAreaItems ) ) {
                         ActionListPressRight( item, mousePos, rtAreaItems.x, rtAreaItems.y + offsetY );
                         return true;
                     }
@@ -526,8 +526,8 @@ namespace Interface
 
         fheroes2::Rect rtAreaItems;
 
-        fheroes2::Button buttonPgUp;
-        fheroes2::Button buttonPgDn;
+        fheroes2::Button _buttonPgUp;
+        fheroes2::Button _buttonPgDn;
 
         fheroes2::Scrollbar _scrollbar;
 
@@ -600,5 +600,3 @@ namespace Interface
         }
     };
 }
-
-#endif
