@@ -1,6 +1,6 @@
 /***************************************************************************
  *   fheroes2: https://github.com/ihhub/fheroes2                           *
- *   Copyright (C) 2020 - 2024                                             *
+ *   Copyright (C) 2020 - 2025                                             *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -204,44 +204,46 @@ bool PrimarySkillsBar::ActionBarLeftMouseSingleClick( int & skill )
     }
 
     // The case when we are in Editor mode.
-    auto primarySkillEditDialog = [&skill]( uint32_t & skillValue ) {
+    auto primarySkillEditDialog = [&skill]( int32_t & skillValue ) {
         std::string header = _( "Set %{skill} Skill" );
         StringReplace( header, "%{skill}", Skill::Primary::String( skill ) );
 
-        return Dialog::SelectCount( header, skill == Skill::Primary::POWER ? 1 : 0, 99, skillValue );
+        const fheroes2::PrimarySkillDialogElement skillUI{ skill, {} };
+
+        return Dialog::SelectCount( std::move( header ), skill == Skill::Primary::POWER ? 1 : 0, 99, skillValue, 1, &skillUI );
     };
 
-    uint32_t value;
+    int32_t value;
 
     switch ( skill ) {
     case Skill::Primary::ATTACK:
-        value = static_cast<uint32_t>( _hero->getAttackBaseValue() );
+        value = _hero->getAttackBaseValue();
         if ( primarySkillEditDialog( value ) ) {
-            _hero->setAttackBaseValue( static_cast<int>( value ) );
+            _hero->setAttackBaseValue( value );
             _isDefault = false;
             return true;
         }
         break;
     case Skill::Primary::DEFENSE:
-        value = static_cast<uint32_t>( _hero->getDefenseBaseValue() );
+        value = _hero->getDefenseBaseValue();
         if ( primarySkillEditDialog( value ) ) {
-            _hero->setDefenseBaseValue( static_cast<int>( value ) );
+            _hero->setDefenseBaseValue( value );
             _isDefault = false;
             return true;
         }
         break;
     case Skill::Primary::POWER:
-        value = static_cast<uint32_t>( _hero->getPowerBaseValue() );
+        value = _hero->getPowerBaseValue();
         if ( primarySkillEditDialog( value ) ) {
-            _hero->setPowerBaseValue( static_cast<int>( value ) );
+            _hero->setPowerBaseValue( value );
             _isDefault = false;
             return true;
         }
         break;
     case Skill::Primary::KNOWLEDGE:
-        value = static_cast<uint32_t>( _hero->getKnowledgeBaseValue() );
+        value = _hero->getKnowledgeBaseValue();
         if ( primarySkillEditDialog( value ) ) {
-            _hero->setKnowledgeBaseValue( static_cast<int>( value ) );
+            _hero->setKnowledgeBaseValue( value );
             _isDefault = false;
             return true;
         }
