@@ -1,6 +1,6 @@
 /***************************************************************************
  *   fheroes2: https://github.com/ihhub/fheroes2                           *
- *   Copyright (C) 2019 - 2024                                             *
+ *   Copyright (C) 2019 - 2025                                             *
  *                                                                         *
  *   Free Heroes2 Engine: http://sourceforge.net/projects/fheroes2         *
  *   Copyright (C) 2009 by Andrey Afletdinov <fheroes2@gmail.com>          *
@@ -23,19 +23,18 @@
 
 #include "heroes.h"
 
+#include <algorithm>
 #include <array>
 #include <cmath>
 #include <cstddef>
 #include <iterator>
 #include <map>
-#include <memory>
-#include <ostream>
 #include <set>
-#include <type_traits>
+#include <sstream>
 #include <utility>
 
 #include "agg_image.h"
-#include "ai.h"
+#include "ai_planner.h"
 #include "army_troop.h"
 #include "artifact.h"
 #include "artifact_info.h"
@@ -43,12 +42,9 @@
 #include "battle.h"
 #include "castle.h"
 #include "dialog.h"
-#include "difficulty.h"
 #include "direction.h"
-#include "game.h"
 #include "game_io.h"
 #include "game_static.h"
-#include "gamedefs.h"
 #include "ground.h"
 #include "icn.h"
 #include "image.h"
@@ -77,7 +73,6 @@
 #include "tools.h"
 #include "translations.h"
 #include "ui_dialog.h"
-#include "ui_text.h"
 #include "world.h"
 
 namespace
@@ -137,7 +132,7 @@ namespace
                         break;
                     }
 
-                    StringAppendModifiers( *strs, GameStatic::ObjectVisitedModifiers( objectType ) );
+                    fheroes2::appendModifierToString( *strs, GameStatic::ObjectVisitedModifiers( objectType ) );
                     strs->append( "\n" );
                 }
             }
@@ -460,7 +455,7 @@ void Heroes::LoadFromMP2( const int32_t mapIndex, const int colorType, const int
     // Reset the army to default
     army.Reset( true );
 
-    StreamBuf dataStream( data );
+    ROStreamBuf dataStream( data );
 
     // Skip first unused byte.
     dataStream.skip( 1 );
@@ -565,7 +560,7 @@ void Heroes::LoadFromMP2( const int32_t mapIndex, const int colorType, const int
     const bool doesHeroHaveCustomName = ( dataStream.get() != 0 );
     if ( doesHeroHaveCustomName ) {
         // An empty name can be set in the original Editor which is wrong.
-        std::string temp = dataStream.toString( 13 );
+        std::string temp = dataStream.getString( 13 );
         if ( !temp.empty() ) {
             SetModes( CUSTOM );
             name = std::move( temp );
@@ -602,10 +597,6 @@ void Heroes::LoadFromMP2( const int32_t mapIndex, const int colorType, const int
     SetSpellPoints( GetMaxSpellPoints() );
     move_point = GetMaxMovePoints();
 
-    if ( isControlAI() ) {
-        AI::Get().HeroesPostLoad( *this );
-    }
-
     DEBUG_LOG( DBG_GAME, DBG_INFO, name << ", color: " << Color::String( GetColor() ) << ", race: " << Race::String( _race ) )
 }
 
@@ -893,7 +884,7 @@ double Heroes::getMeetingValue( const Heroes & receivingHero ) const
 
     // Magic Book is not transferable.
     const uint32_t artCount = bag_artifacts.CountArtifacts() - bag_artifacts.Count( Artifact::MAGIC_BOOK );
-    const uint32_t canFit = HEROESMAXARTIFACT - receivingHero.bag_artifacts.CountArtifacts();
+    const uint32_t canFit = BagArtifacts::maxCapacity - receivingHero.bag_artifacts.CountArtifacts();
 
     double artifactValue = bag_artifacts.getArtifactValue() * 5.0;
     if ( artCount > canFit ) {
@@ -912,7 +903,7 @@ int Heroes::GetAttack() const
 int Heroes::GetAttack( std::string * strs ) const
 {
     int result = attack + GetAttackModificator( strs );
-    return result < 0 ? 0 : ( result > 255 ? 255 : result );
+    return std::clamp( result, 0, 255 );
 }
 
 int Heroes::GetDefense() const
@@ -923,7 +914,7 @@ int Heroes::GetDefense() const
 int Heroes::GetDefense( std::string * strs ) const
 {
     int result = defense + GetDefenseModificator( strs );
-    return result < 0 ? 0 : ( result > 255 ? 255 : result );
+    return std::clamp( result, 0, 255 );
 }
 
 int Heroes::GetPower() const
@@ -934,7 +925,7 @@ int Heroes::GetPower() const
 int Heroes::GetPower( std::string * strs ) const
 {
     const int result = power + GetPowerModificator( strs );
-    return result < 1 ? 1 : ( result > 255 ? 255 : result );
+    return std::clamp( result, 1, 255 );
 }
 
 int Heroes::GetKnowledge() const
@@ -945,7 +936,7 @@ int Heroes::GetKnowledge() const
 int Heroes::GetKnowledge( std::string * strs ) const
 {
     int result = knowledge + GetKnowledgeModificator( strs );
-    return result < 0 ? 0 : ( result > 255 ? 255 : result );
+    return std::clamp( result, 0, 255 );
 }
 
 void Heroes::IncreasePrimarySkill( int skill )
@@ -1036,11 +1027,6 @@ uint32_t Heroes::GetMaxMovePoints( const bool onWater ) const
         }
     }
 
-    // AI-controlled heroes receive additional movement bonus depending on the game difficulty
-    if ( isControlAI() ) {
-        result += Difficulty::GetHeroMovementBonusForAI( Game::getDifficulty() );
-    }
-
     return result;
 }
 
@@ -1146,7 +1132,7 @@ bool Heroes::Recruit( const int col, const fheroes2::Point & pt )
         army.Reset( false );
     }
 
-    world.GetTiles( pt.x, pt.y ).setHero( this );
+    world.getTile( pt.x, pt.y ).setHero( this );
 
     kingdom.AddHero( this );
     // Update the set of recruits in the kingdom
@@ -1213,7 +1199,7 @@ uint32_t Heroes::getDailyRestoredSpellPoints() const
     // Spell points from artifacts.
     points += static_cast<uint32_t>( GetBagArtifacts().getTotalArtifactEffectValue( fheroes2::ArtifactBonusType::SPELL_POINTS_DAILY_GENERATION ) );
 
-    points += GetSecondaryValues( Skill::Secondary::MYSTICISM );
+    points += GetSecondarySkillValue( Skill::Secondary::MYSTICISM );
 
     return points;
 }
@@ -1278,10 +1264,10 @@ Castle * Heroes::inCastleMutable() const
     return castle && castle->GetHero() == this ? castle : nullptr;
 }
 
-bool Heroes::isVisited( const Maps::Tiles & tile, Visit::type_t type ) const
+bool Heroes::isVisited( const Maps::Tile & tile, Visit::Type type ) const
 {
     const int32_t index = tile.GetIndex();
-    const MP2::MapObjectType objectType = tile.GetObject( false );
+    const MP2::MapObjectType objectType = tile.getMainObjectType( false );
 
     if ( Visit::GLOBAL == type ) {
         return GetKingdom().isVisited( index, objectType );
@@ -1290,7 +1276,7 @@ bool Heroes::isVisited( const Maps::Tiles & tile, Visit::type_t type ) const
     return visit_object.end() != std::find( visit_object.begin(), visit_object.end(), IndexObject( index, objectType ) );
 }
 
-bool Heroes::isObjectTypeVisited( const MP2::MapObjectType objectType, Visit::type_t type ) const
+bool Heroes::isObjectTypeVisited( const MP2::MapObjectType objectType, Visit::Type type ) const
 {
     if ( Visit::GLOBAL == type ) {
         return GetKingdom().isVisited( objectType );
@@ -1299,10 +1285,10 @@ bool Heroes::isObjectTypeVisited( const MP2::MapObjectType objectType, Visit::ty
     return std::any_of( visit_object.begin(), visit_object.end(), [objectType]( const IndexObject & v ) { return v.isObject( objectType ); } );
 }
 
-void Heroes::SetVisited( int32_t index, Visit::type_t type )
+void Heroes::SetVisited( int32_t index, Visit::Type type )
 {
-    const Maps::Tiles & tile = world.GetTiles( index );
-    const MP2::MapObjectType objectType = tile.GetObject( false );
+    const Maps::Tile & tile = world.getTile( index );
+    const MP2::MapObjectType objectType = tile.getMainObjectType( false );
 
     if ( Visit::GLOBAL == type ) {
         GetKingdom().SetVisited( index, objectType );
@@ -1314,8 +1300,8 @@ void Heroes::SetVisited( int32_t index, Visit::type_t type )
 
 void Heroes::setVisitedForAllies( const int32_t tileIndex ) const
 {
-    const Maps::Tiles & tile = world.GetTiles( tileIndex );
-    const MP2::MapObjectType objectType = tile.GetObject( false );
+    const Maps::Tile & tile = world.getTile( tileIndex );
+    const MP2::MapObjectType objectType = tile.getMainObjectType( false );
 
     // Set visited to all allies as well.
     const Colors friendColors( Players::GetPlayerFriends( GetColor() ) );
@@ -1324,10 +1310,10 @@ void Heroes::setVisitedForAllies( const int32_t tileIndex ) const
     }
 }
 
-void Heroes::SetVisitedWideTile( int32_t index, const MP2::MapObjectType objectType, Visit::type_t type )
+void Heroes::SetVisitedWideTile( int32_t index, const MP2::MapObjectType objectType, Visit::Type type )
 {
-    const Maps::Tiles & tile = world.GetTiles( index );
-    const uint32_t uid = tile.GetObjectUID();
+    const Maps::Tile & tile = world.getTile( index );
+    const uint32_t uid = tile.getMainObjectPart()._uid;
     int wide = 0;
 
     switch ( objectType ) {
@@ -1344,9 +1330,9 @@ void Heroes::SetVisitedWideTile( int32_t index, const MP2::MapObjectType objectT
         break;
     }
 
-    if ( tile.GetObject( false ) == objectType && wide ) {
+    if ( tile.getMainObjectType( false ) == objectType && wide ) {
         for ( int32_t ii = tile.GetIndex() - ( wide - 1 ); ii <= tile.GetIndex() + ( wide - 1 ); ++ii )
-            if ( Maps::isValidAbsIndex( ii ) && world.GetTiles( ii ).GetObjectUID() == uid )
+            if ( Maps::isValidAbsIndex( ii ) && world.getTile( ii ).getMainObjectPart()._uid == uid )
                 SetVisited( ii, type );
     }
 }
@@ -1584,6 +1570,11 @@ uint32_t Heroes::GetExperienceFromLevel( int lvl )
     return ( l1 + static_cast<uint32_t>( round( ( l1 - GetExperienceFromLevel( lvl - 2 ) ) * 1.2 / 100 ) * 100 ) );
 }
 
+uint32_t Heroes::getExperienceMaxValue()
+{
+    return 2990600;
+}
+
 bool Heroes::BuySpellBook( const Castle * castle )
 {
     if ( HaveSpellBook() || Color::NONE == GetColor() ) {
@@ -1602,8 +1593,7 @@ bool Heroes::BuySpellBook( const Castle * castle )
             header.append( _( "Unfortunately, you seem to be a little short of cash at the moment." ) );
 
             const fheroes2::ArtifactDialogElement artifactUI( Artifact::MAGIC_BOOK );
-            fheroes2::showMessage( fheroes2::Text( GetName(), fheroes2::FontType::normalYellow() ), fheroes2::Text( header, fheroes2::FontType::normalWhite() ),
-                                   Dialog::OK, { &artifactUI } );
+            fheroes2::showStandardTextMessage( GetName(), std::move( header ), Dialog::OK, { &artifactUI } );
         }
         return false;
     }
@@ -1613,9 +1603,7 @@ bool Heroes::BuySpellBook( const Castle * castle )
         header.append( _( "Do you wish to buy one?" ) );
 
         const fheroes2::ArtifactDialogElement artifactUI( Artifact::MAGIC_BOOK );
-        if ( fheroes2::showMessage( fheroes2::Text( GetName(), fheroes2::FontType::normalYellow() ), fheroes2::Text( header, fheroes2::FontType::normalWhite() ),
-                                    Dialog::YES | Dialog::NO, { &artifactUI } )
-             == Dialog::NO ) {
+        if ( fheroes2::showStandardTextMessage( GetName(), std::move( header ), Dialog::YES | Dialog::NO, { &artifactUI } ) == Dialog::NO ) {
             return false;
         }
     }
@@ -1640,7 +1628,7 @@ bool Heroes::isMoveEnabled() const
 
 bool Heroes::CanMove() const
 {
-    const Maps::Tiles & tile = world.GetTiles( GetIndex() );
+    const Maps::Tile & tile = world.getTile( GetIndex() );
     return move_point >= ( tile.isRoad() ? Maps::Ground::roadPenalty : Maps::Ground::GetPenalty( tile, GetLevelSkill( Skill::Secondary::PATHFINDING ) ) );
 }
 
@@ -1654,7 +1642,7 @@ void Heroes::SetMove( const bool enable )
         ResetModes( SLEEPER );
 
         if ( isControlAI() ) {
-            AI::Get().HeroesBeginMovement( *this );
+            AI::Planner::Get().HeroesBeginMovement( *this );
         }
 
         SetModes( ENABLEMOVE );
@@ -1666,10 +1654,6 @@ void Heroes::SetMove( const bool enable )
 
         ResetModes( ENABLEMOVE );
 
-        if ( isControlAI() ) {
-            AI::Get().HeroesFinishMovement( *this );
-        }
-
         // Reset the hero sprite
         resetHeroSprite();
     }
@@ -1716,14 +1700,14 @@ bool Heroes::HasSecondarySkill( int skill ) const
     return Skill::Level::NONE != secondary_skills.GetLevel( skill );
 }
 
-uint32_t Heroes::GetSecondaryValues( int skill ) const
+uint32_t Heroes::GetSecondarySkillValue( int skill ) const
 {
-    return secondary_skills.GetValues( skill );
+    return secondary_skills.GetValue( skill );
 }
 
 bool Heroes::HasMaxSecondarySkill() const
 {
-    return HEROESMAXSKILL <= secondary_skills.Count();
+    return maxNumOfSecSkills <= secondary_skills.Count();
 }
 
 int Heroes::GetLevelSkill( int skill ) const
@@ -1761,7 +1745,7 @@ void Heroes::Scout( const int tileIndex ) const
 int Heroes::GetScoutingDistance() const
 {
     return static_cast<int>( GetBagArtifacts().getTotalArtifactEffectValue( fheroes2::ArtifactBonusType::AREA_REVEAL_DISTANCE )
-                             + GameStatic::getFogDiscoveryDistance( GameStatic::FogDiscoveryType::HEROES ) + GetSecondaryValues( Skill::Secondary::SCOUTING ) );
+                             + GameStatic::getFogDiscoveryDistance( GameStatic::FogDiscoveryType::HEROES ) + GetSecondarySkillValue( Skill::Secondary::SCOUTING ) );
 }
 
 fheroes2::Rect Heroes::GetScoutRoi() const
@@ -1778,7 +1762,7 @@ uint32_t Heroes::UpdateMovementPoints( const uint32_t movePoints, const int skil
     if ( level == Skill::Level::NONE )
         return movePoints;
 
-    const uint32_t skillValue = GetSecondaryValues( skill );
+    const uint32_t skillValue = GetSecondarySkillValue( skill );
 
     if ( skillValue == 33 ) {
         return movePoints * 4 / 3;
@@ -1846,10 +1830,9 @@ void Heroes::LevelUp( bool skipsecondary, bool autoselect )
 
     DEBUG_LOG( DBG_GAME, DBG_INFO, "for " << GetName() << ", up " << Skill::Primary::String( primarySkill ) )
 
-    if ( !skipsecondary )
-        LevelUpSecondarySkill( seeds, primarySkill, ( autoselect || isControlAI() ) );
-    if ( isControlAI() )
-        AI::Get().HeroesLevelUp( *this );
+    if ( !skipsecondary ) {
+        LevelUpSecondarySkill( seeds, primarySkill, autoselect );
+    }
 }
 
 void Heroes::LevelUpSecondarySkill( const HeroSeedsForLevelUp & seeds, int primary, bool autoselect )
@@ -1876,6 +1859,9 @@ void Heroes::LevelUpSecondarySkill( const HeroSeedsForLevelUp & seeds, int prima
             selected = sec1.isValid() ? sec1 : sec2;
         }
     }
+    else if ( isControlAI() ) {
+        selected = AI::Planner::pickSecondarySkill( *this, sec1, sec2 );
+    }
     else {
         AudioManager::PlaySound( M82::NWHEROLV );
         const int result = Dialog::LevelUpSelectSkill( name, primary, sec1, sec2, *this );
@@ -1925,7 +1911,7 @@ bool Heroes::MayStillMove( const bool ignorePath, const bool ignoreSleeper ) con
         return false;
     }
 
-    if ( path.isValidForMovement() && !ignorePath ) {
+    if ( !ignorePath && path.isValidForMovement() ) {
         return path.hasAllowedSteps();
     }
 
@@ -1971,7 +1957,7 @@ void Heroes::Dismiss( int reason )
     }
     SetColor( Color::NONE );
 
-    world.GetTiles( GetIndex() ).setHero( nullptr );
+    world.getTile( GetIndex() ).setHero( nullptr );
     SetIndex( -1 );
 
     modes = 0;
@@ -2030,8 +2016,9 @@ void Heroes::ActionNewPosition( const bool allowMonsterAttack )
         }
     }
 
-    if ( isControlAI() )
-        AI::Get().HeroesActionNewPosition( *this );
+    if ( isControlAI() ) {
+        AI::Planner::Get().HeroesActionNewPosition( *this );
+    }
 
     ResetModes( VISIONS );
 }
@@ -2044,9 +2031,9 @@ void Heroes::Move2Dest( const int32_t dstIndex )
         return;
     }
 
-    world.GetTiles( currentIndex ).setHero( nullptr );
+    world.getTile( currentIndex ).setHero( nullptr );
     SetIndex( dstIndex );
-    world.GetTiles( dstIndex ).setHero( this );
+    world.getTile( dstIndex ).setHero( this );
 }
 
 const fheroes2::Sprite & Heroes::GetPortrait( int id, int type )
@@ -2054,7 +2041,7 @@ const fheroes2::Sprite & Heroes::GetPortrait( int id, int type )
     if ( isValidId( id ) )
         switch ( type ) {
         case PORT_BIG:
-            return fheroes2::AGG::GetICN( ICN::PORTxxxx( id ), 0 );
+            return fheroes2::AGG::GetICN( ICN::getHeroPortraitIcnId( id ), 0 );
         case PORT_MEDIUM: {
             // Original ICN::PORTMEDI sprites are badly rendered. Instead of them we're getting high quality ICN:PORT00xx file and resize it to a smaller image.
             // TODO: find a better way to store these images, ideally in agg_image.cpp file.
@@ -2064,7 +2051,7 @@ const fheroes2::Sprite & Heroes::GetPortrait( int id, int type )
                 return iter->second;
             }
 
-            const fheroes2::Sprite & original = fheroes2::AGG::GetICN( ICN::PORTxxxx( id ), 0 );
+            const fheroes2::Sprite & original = fheroes2::AGG::GetICN( ICN::getHeroPortraitIcnId( id ), 0 );
             fheroes2::Sprite output( 50, 47 );
             fheroes2::Resize( original, output );
 
@@ -2151,7 +2138,7 @@ std::string Heroes::String() const
        << "direction       : " << Direction::String( direction ) << std::endl
        << "index sprite    : " << sprite_index << std::endl
        << "in castle       : " << ( inCastle() ? "true" : "false" ) << std::endl
-       << "save object     : " << MP2::StringObject( world.GetTiles( GetIndex() ).GetObject( false ) ) << std::endl
+       << "save object     : " << MP2::StringObject( world.getTile( GetIndex() ).getMainObjectType( false ) ) << std::endl
        << "flags           : " << ( Modes( SHIPMASTER ) ? "SHIPMASTER," : "" ) << ( Modes( CUSTOM ) ? "CUSTOM," : "" ) << ( Modes( PATROL ) ? "PATROL" : "" )
        << std::endl;
 
@@ -2162,7 +2149,7 @@ std::string Heroes::String() const
     if ( !visit_object.empty() ) {
         os << "visit objects   : ";
         for ( const auto & info : visit_object ) {
-            os << MP2::StringObject( static_cast<MP2::MapObjectType>( info.second ) ) << "(" << info.first << "), ";
+            os << MP2::StringObject( info.second ) << "(" << info.first << "), ";
         }
 
         os << std::endl;
@@ -2174,8 +2161,6 @@ std::string Heroes::String() const
            << "spell book      : " << ( HaveSpellBook() ? spell_book.String() : "disabled" ) << std::endl
            << "army dump       : " << army.String() << std::endl
            << "ai role         : " << GetHeroRoleString( *this ) << std::endl;
-
-        os << AI::Get().HeroesString( *this );
     }
 
     return os.str();
@@ -2245,98 +2230,61 @@ double Heroes::getAIMinimumJoiningArmyStrength() const
     return strengthThreshold * Troops( GetArmy().getTroops() ).GetStrength();
 }
 
-AllHeroes::~AllHeroes()
-{
-    clear();
-}
-
 void AllHeroes::Init()
 {
-    clear();
+    Clear();
 
-    reserve( Heroes::HEROES_COUNT );
+    _heroes.reserve( Heroes::HEROES_COUNT );
 
-    push_back( new Heroes( Heroes::UNKNOWN, Race::KNGT ) );
+    _heroes.emplace_back( std::make_unique<Heroes>( Heroes::UNKNOWN, Race::KNGT ) );
 
     for ( const int race : std::array<int, 6>{ Race::KNGT, Race::BARB, Race::SORC, Race::WRLK, Race::WZRD, Race::NECR } ) {
         const auto [minHeroId, maxHeroId] = getHeroIdRangeForRace( race );
         assert( minHeroId <= maxHeroId );
 
         for ( int hid = minHeroId; hid <= maxHeroId; ++hid ) {
-            push_back( new Heroes( hid, race ) );
+            _heroes.emplace_back( std::make_unique<Heroes>( hid, race ) );
         }
     }
 
     // SW campaign
-    push_back( new Heroes( Heroes::ROLAND, Race::WZRD, 5000 ) );
-    push_back( new Heroes( Heroes::CORLAGON, Race::KNGT, 5000 ) );
-    push_back( new Heroes( Heroes::ELIZA, Race::SORC, 5000 ) );
-    push_back( new Heroes( Heroes::ARCHIBALD, Race::WRLK, 5000 ) );
-    push_back( new Heroes( Heroes::HALTON, Race::KNGT, 5000 ) );
-    push_back( new Heroes( Heroes::BRAX, Race::NECR, 5000 ) );
+    _heroes.emplace_back( std::make_unique<Heroes>( Heroes::ROLAND, Race::WZRD, 5000 ) );
+    _heroes.emplace_back( std::make_unique<Heroes>( Heroes::CORLAGON, Race::KNGT, 5000 ) );
+    _heroes.emplace_back( std::make_unique<Heroes>( Heroes::ELIZA, Race::SORC, 5000 ) );
+    _heroes.emplace_back( std::make_unique<Heroes>( Heroes::ARCHIBALD, Race::WRLK, 5000 ) );
+    _heroes.emplace_back( std::make_unique<Heroes>( Heroes::HALTON, Race::KNGT, 5000 ) );
+    _heroes.emplace_back( std::make_unique<Heroes>( Heroes::BRAX, Race::NECR, 5000 ) );
 
     // PoL
     const GameVersion version = Settings::Get().getCurrentMapInfo().version;
     if ( version == GameVersion::PRICE_OF_LOYALTY || version == GameVersion::RESURRECTION ) {
-        push_back( new Heroes( Heroes::SOLMYR, Race::WZRD, 5000 ) );
-        push_back( new Heroes( Heroes::DAINWIN, Race::WRLK, 5000 ) );
-        push_back( new Heroes( Heroes::MOG, Race::NECR, 5000 ) );
-        push_back( new Heroes( Heroes::UNCLEIVAN, Race::BARB, 5000 ) );
-        push_back( new Heroes( Heroes::JOSEPH, Race::WZRD, 5000 ) );
-        push_back( new Heroes( Heroes::GALLAVANT, Race::KNGT, 5000 ) );
-        push_back( new Heroes( Heroes::ELDERIAN, Race::WRLK, 5000 ) );
-        push_back( new Heroes( Heroes::CEALLACH, Race::KNGT, 5000 ) );
-        push_back( new Heroes( Heroes::DRAKONIA, Race::WZRD, 5000 ) );
-        push_back( new Heroes( Heroes::MARTINE, Race::SORC, 5000 ) );
-        push_back( new Heroes( Heroes::JARKONAS, Race::BARB, 5000 ) );
+        _heroes.emplace_back( std::make_unique<Heroes>( Heroes::SOLMYR, Race::WZRD, 5000 ) );
+        _heroes.emplace_back( std::make_unique<Heroes>( Heroes::DAINWIN, Race::WRLK, 5000 ) );
+        _heroes.emplace_back( std::make_unique<Heroes>( Heroes::MOG, Race::NECR, 5000 ) );
+        _heroes.emplace_back( std::make_unique<Heroes>( Heroes::UNCLEIVAN, Race::BARB, 5000 ) );
+        _heroes.emplace_back( std::make_unique<Heroes>( Heroes::JOSEPH, Race::WZRD, 5000 ) );
+        _heroes.emplace_back( std::make_unique<Heroes>( Heroes::GALLAVANT, Race::KNGT, 5000 ) );
+        _heroes.emplace_back( std::make_unique<Heroes>( Heroes::ELDERIAN, Race::WRLK, 5000 ) );
+        _heroes.emplace_back( std::make_unique<Heroes>( Heroes::CEALLACH, Race::KNGT, 5000 ) );
+        _heroes.emplace_back( std::make_unique<Heroes>( Heroes::DRAKONIA, Race::WZRD, 5000 ) );
+        _heroes.emplace_back( std::make_unique<Heroes>( Heroes::MARTINE, Race::SORC, 5000 ) );
+        _heroes.emplace_back( std::make_unique<Heroes>( Heroes::JARKONAS, Race::BARB, 5000 ) );
     }
     else {
         // for non-PoL maps, just add unknown heroes instead in place of the PoL-specific ones
         for ( int i = Heroes::SOLMYR; i <= Heroes::JARKONAS; ++i ) {
-            push_back( new Heroes( Heroes::UNKNOWN, Race::KNGT ) );
+            _heroes.emplace_back( std::make_unique<Heroes>( Heroes::UNKNOWN, Race::KNGT ) );
         }
     }
 
     if ( IS_DEVEL() ) {
-        push_back( new Heroes( Heroes::DEBUG_HERO, Race::WRLK ) );
+        _heroes.emplace_back( std::make_unique<Heroes>( Heroes::DEBUG_HERO, Race::WRLK ) );
     }
     else {
-        push_back( new Heroes( Heroes::UNKNOWN, Race::KNGT ) );
+        _heroes.emplace_back( std::make_unique<Heroes>( Heroes::UNKNOWN, Race::KNGT ) );
     }
 
-    assert( size() == Heroes::HEROES_COUNT );
-}
-
-void AllHeroes::clear()
-{
-    std::for_each( begin(), end(), []( Heroes * hero ) {
-        assert( hero != nullptr );
-
-        delete hero;
-    } );
-
-    std::vector<Heroes *>::clear();
-}
-
-Heroes * VecHeroes::Get( int hid ) const
-{
-    if ( !Heroes::isValidId( hid ) ) {
-        return nullptr;
-    }
-
-    return ( *this )[hid];
-}
-
-Heroes * VecHeroes::Get( const fheroes2::Point & center ) const
-{
-    for ( Heroes * hero : *this ) {
-        assert( hero != nullptr );
-        if ( hero->isPosition( center ) ) {
-            return hero;
-        }
-    }
-
-    return nullptr;
+    assert( _heroes.size() == Heroes::HEROES_COUNT );
 }
 
 Heroes * AllHeroes::GetHeroForHire( const int race, const int heroIDToIgnore ) const
@@ -2414,31 +2362,98 @@ Heroes * AllHeroes::GetHeroForHire( const int race, const int heroIDToIgnore ) c
     std::vector<int> heroesForHireNotRecruits = heroesForHire;
 
     heroesForHireNotRecruits.erase( std::remove_if( heroesForHireNotRecruits.begin(), heroesForHireNotRecruits.end(),
-                                                    [this]( const int heroID ) { return at( heroID )->Modes( Heroes::RECRUIT ); } ),
+                                                    [this]( const int heroID ) {
+                                                        assert( heroID >= 0 && static_cast<size_t>( heroID ) < _heroes.size() && _heroes[heroID] );
+
+                                                        return _heroes[heroID]->Modes( Heroes::RECRUIT );
+                                                    } ),
                                     heroesForHireNotRecruits.end() );
 
-    if ( !heroesForHireNotRecruits.empty() ) {
-        return at( Rand::Get( heroesForHireNotRecruits ) );
+    // If there are no heroes who are not yet available for recruitment, then allow heroes to be available for recruitment in several kingdoms at the same time
+    const int heroID = heroesForHireNotRecruits.empty() ? Rand::Get( heroesForHire ) : Rand::Get( heroesForHireNotRecruits );
+    assert( heroID >= 0 && static_cast<size_t>( heroID ) < _heroes.size() && _heroes[heroID] );
+
+    return _heroes[heroID].get();
+}
+
+Heroes * AllHeroes::Get( const int hid ) const
+{
+    if ( !Heroes::isValidId( hid ) ) {
+        return nullptr;
     }
 
-    // There are no heroes who are not yet available for recruitment, allow heroes to be available for recruitment in several kingdoms at the same time
-    return at( Rand::Get( heroesForHire ) );
+    assert( hid >= 0 && static_cast<size_t>( hid ) < _heroes.size() && _heroes[hid] );
+
+    return _heroes[hid].get();
+}
+
+Heroes * AllHeroes::Get( const fheroes2::Point & center ) const
+{
+    for ( Heroes * hero : *this ) {
+        assert( hero != nullptr );
+
+        if ( hero->isPosition( center ) ) {
+            return hero;
+        }
+    }
+
+    return nullptr;
 }
 
 void AllHeroes::Scout( int colors ) const
 {
     for ( const Heroes * hero : *this ) {
         assert( hero != nullptr );
-        if ( colors & hero->GetColor() ) {
-            hero->Scout( hero->GetIndex() );
+
+        if ( !( hero->GetColor() & colors ) ) {
+            continue;
         }
+
+        hero->Scout( hero->GetIndex() );
     }
 }
 
+void AllHeroes::ResetModes( const uint32_t modes ) const
+{
+    std::for_each( begin(), end(), [modes]( Heroes * hero ) {
+        assert( hero != nullptr );
+
+        hero->ResetModes( modes );
+    } );
+}
+
+void AllHeroes::NewDay() const
+{
+    std::for_each( begin(), end(), []( Heroes * hero ) {
+        assert( hero != nullptr );
+
+        hero->ActionNewDay();
+    } );
+}
+
+void AllHeroes::NewWeek() const
+{
+    std::for_each( begin(), end(), []( Heroes * hero ) {
+        assert( hero != nullptr );
+
+        hero->ActionNewWeek();
+    } );
+}
+
+void AllHeroes::NewMonth() const
+{
+    std::for_each( begin(), end(), []( Heroes * hero ) {
+        assert( hero != nullptr );
+
+        hero->ActionNewMonth();
+    } );
+}
+
 Heroes * AllHeroes::FromJail( int32_t index ) const
 {
     for ( Heroes * hero : *this ) {
         assert( hero != nullptr );
+
         if ( hero->Modes( Heroes::JAIL ) && index == hero->GetIndex() ) {
             return hero;
         }
@@ -2447,39 +2462,45 @@ Heroes * AllHeroes::FromJail( int32_t index ) const
     return nullptr;
 }
 
-StreamBase & operator<<( StreamBase & msg, const VecHeroes & heroes )
+OStreamBase & operator<<( OStreamBase & stream, const VecHeroes & heroes )
 {
-    msg << static_cast<uint32_t>( heroes.size() );
+    stream.put32( static_cast<uint32_t>( heroes.size() ) );
 
-    for ( const Heroes * hero : heroes ) {
+    std::for_each( heroes.begin(), heroes.end(), [&stream]( const Heroes * hero ) {
         assert( hero != nullptr );
-        msg << hero->GetID();
-    }
 
-    return msg;
+        stream << hero->GetID();
+    } );
+
+    return stream;
 }
 
-StreamBase & operator>>( StreamBase & msg, VecHeroes & heroes )
+IStreamBase & operator>>( IStreamBase & stream, VecHeroes & heroes )
 {
-    uint32_t size;
-    msg >> size;
+    const uint32_t size = stream.get32();
 
     heroes.clear();
+    heroes.reserve( size );
 
     for ( uint32_t i = 0; i < size; ++i ) {
-        int32_t hid;
-        msg >> hid;
+        int32_t hid{ -1 };
+        stream >> hid;
 
         static_assert( LAST_SUPPORTED_FORMAT_VERSION < FORMAT_VERSION_1010_RELEASE, "Remove the logic below." );
         if ( Game::GetVersionOfCurrentSaveFile() < FORMAT_VERSION_1010_RELEASE ) {
             // UNKNOWN was 72 before FORMAT_VERSION_1010_RELEASE. UNKNOWN hero shouldn't exist!
             if ( hid == 72 || !Heroes::isValidId( hid + 1 ) ) {
+                // Most likely the save file is corrupted.
+                stream.setFail();
+
                 continue;
             }
 
             Heroes * hero = world.GetHeroes( hid + 1 );
             if ( hero == nullptr ) {
-                // Most likely save file is corrupted.
+                // Most likely the save file is corrupted.
+                stream.setFail();
+
                 continue;
             }
 
@@ -2487,12 +2508,17 @@ StreamBase & operator>>( StreamBase & msg, VecHeroes & heroes )
         }
         else {
             if ( !Heroes::isValidId( hid ) ) {
+                // Most likely the save file is corrupted.
+                stream.setFail();
+
                 continue;
             }
 
             Heroes * hero = world.GetHeroes( hid );
             if ( hero == nullptr ) {
-                // Most likely save file is corrupted.
+                // Most likely the save file is corrupted.
+                stream.setFail();
+
                 continue;
             }
 
@@ -2500,35 +2526,32 @@ StreamBase & operator>>( StreamBase & msg, VecHeroes & heroes )
         }
     }
 
-    return msg;
+    return stream;
 }
 
-StreamBase & operator<<( StreamBase & msg, const Heroes & hero )
+OStreamBase & operator<<( OStreamBase & stream, const Heroes & hero )
 {
     const HeroBase & base = hero;
     const ColorBase & col = hero;
 
     // HeroBase
-    msg << base;
+    stream << base;
 
     // Heroes
-    using ObjectTypeUnderHeroType = std::underlying_type_t<decltype( hero._objectTypeUnderHero )>;
-
-    return msg << hero.name << col << hero.experience << hero.secondary_skills << hero.army << hero._id << hero.portrait << hero._race
-               << static_cast<ObjectTypeUnderHeroType>( hero._objectTypeUnderHero ) << hero.path << hero.direction << hero.sprite_index << hero._patrolCenter
-               << hero._patrolDistance << hero.visit_object << hero._lastGroundRegion;
+    return stream << hero.name << col << hero.experience << hero.secondary_skills << hero.army << hero._id << hero.portrait << hero._race << hero._objectTypeUnderHero
+                  << hero.path << hero.direction << hero.sprite_index << hero._patrolCenter << hero._patrolDistance << hero.visit_object << hero._lastGroundRegion;
 }
 
-StreamBase & operator>>( StreamBase & msg, Heroes & hero )
+IStreamBase & operator>>( IStreamBase & stream, Heroes & hero )
 {
     HeroBase & base = hero;
     ColorBase & col = hero;
 
     // HeroBase
-    msg >> base;
+    stream >> base;
 
     // Heroes
-    msg >> hero.name >> col >> hero.experience >> hero.secondary_skills >> hero.army >> hero._id >> hero.portrait >> hero._race;
+    stream >> hero.name >> col >> hero.experience >> hero.secondary_skills >> hero.army >> hero._id >> hero.portrait >> hero._race;
 
     static_assert( LAST_SUPPORTED_FORMAT_VERSION < FORMAT_VERSION_PRE1_1100_RELEASE, "Remove the logic below." );
     if ( Game::GetVersionOfCurrentSaveFile() < FORMAT_VERSION_PRE1_1100_RELEASE ) {
@@ -2552,91 +2575,95 @@ StreamBase & operator>>( StreamBase & msg, Heroes & hero )
         }
     }
 
-    using ObjectTypeUnderHeroType = std::underlying_type_t<decltype( hero._objectTypeUnderHero )>;
-    static_assert( std::is_same_v<ObjectTypeUnderHeroType, uint16_t>, "Type of _objectTypeUnderHero has been changed, check the logic below." );
-
     static_assert( LAST_SUPPORTED_FORMAT_VERSION < FORMAT_VERSION_PRE3_1100_RELEASE, "Remove the logic below." );
     if ( Game::GetVersionOfCurrentSaveFile() < FORMAT_VERSION_PRE3_1100_RELEASE ) {
         static_assert( LAST_SUPPORTED_FORMAT_VERSION < FORMAT_VERSION_PRE1_1009_RELEASE, "Remove the logic below." );
         if ( Game::GetVersionOfCurrentSaveFile() < FORMAT_VERSION_PRE1_1009_RELEASE ) {
             int temp = 0;
-            msg >> temp;
+            stream >> temp;
 
             hero._objectTypeUnderHero = static_cast<MP2::MapObjectType>( temp );
         }
         else {
             uint8_t temp = 0;
 
-            msg >> temp;
+            stream >> temp;
 
             hero._objectTypeUnderHero = static_cast<MP2::MapObjectType>( temp );
         }
     }
     else {
-        ObjectTypeUnderHeroType temp = 0;
-
-        msg >> temp;
-
-        hero._objectTypeUnderHero = static_cast<MP2::MapObjectType>( temp );
+        stream >> hero._objectTypeUnderHero;
     }
 
-    msg >> hero.path >> hero.direction >> hero.sprite_index;
+    stream >> hero.path >> hero.direction >> hero.sprite_index;
 
     static_assert( LAST_SUPPORTED_FORMAT_VERSION < FORMAT_VERSION_1010_RELEASE, "Remove the logic below." );
     if ( Game::GetVersionOfCurrentSaveFile() < FORMAT_VERSION_1010_RELEASE ) {
         int16_t patrolX = 0;
         int16_t patrolY = 0;
 
-        msg >> patrolX >> patrolY;
+        stream >> patrolX >> patrolY;
         hero._patrolCenter = fheroes2::Point( patrolX, patrolY );
     }
     else {
-        msg >> hero._patrolCenter;
+        stream >> hero._patrolCenter;
     }
 
-    msg >> hero._patrolDistance >> hero.visit_object >> hero._lastGroundRegion;
+    stream >> hero._patrolDistance >> hero.visit_object >> hero._lastGroundRegion;
 
     hero.army.SetCommander( &hero );
-    return msg;
+
+    return stream;
 }
 
-StreamBase & operator<<( StreamBase & msg, const AllHeroes & heroes )
+OStreamBase & operator<<( OStreamBase & stream, const AllHeroes & heroes )
 {
-    msg << static_cast<uint32_t>( heroes.size() );
+    stream.put32( static_cast<uint32_t>( heroes._heroes.size() ) );
 
-    for ( Heroes * const & hero : heroes ) {
-        msg << *hero;
-    }
+    std::for_each( heroes.begin(), heroes.end(), [&stream]( const Heroes * hero ) {
+        assert( hero != nullptr );
 
-    return msg;
+        stream << *hero;
+    } );
+
+    return stream;
 }
 
-StreamBase & operator>>( StreamBase & msg, AllHeroes & heroes )
+IStreamBase & operator>>( IStreamBase & stream, AllHeroes & heroes )
 {
-    uint32_t size;
-    msg >> size;
+    std::vector<std::unique_ptr<Heroes>> & heroesRef = heroes._heroes;
 
-    heroes.clear();
-    heroes.resize( size, nullptr );
+    const uint32_t size = stream.get32();
+
+    heroesRef.clear();
+    heroesRef.reserve( size );
 
     static_assert( LAST_SUPPORTED_FORMAT_VERSION < FORMAT_VERSION_1010_RELEASE, "Remove the logic below." );
     if ( Game::GetVersionOfCurrentSaveFile() < FORMAT_VERSION_1010_RELEASE ) {
         // Before FORMAT_VERSION_1010_RELEASE UNKNOWN hero was last while now it is first.
         // In order to preserve the original order of heroes we have to do the below trick.
-        for ( size_t i = 1; i < heroes.size(); ++i ) {
-            heroes[i] = new Heroes();
-            msg >> *heroes[i];
-        }
+        if ( size > 0 ) {
+            heroesRef.emplace_back( std::make_unique<Heroes>() );
 
-        heroes[0] = new Heroes();
-        msg >> *heroes[0];
+            for ( uint32_t i = 1; i < size; ++i ) {
+                auto hero = std::make_unique<Heroes>();
+                stream >> *hero;
+
+                heroesRef.emplace_back( std::move( hero ) );
+            }
+
+            stream >> *( heroesRef[0] );
+        }
     }
     else {
-        for ( Heroes *& hero : heroes ) {
-            hero = new Heroes();
-            msg >> *hero;
+        for ( uint32_t i = 0; i < size; ++i ) {
+            auto hero = std::make_unique<Heroes>();
+            stream >> *hero;
+
+            heroesRef.emplace_back( std::move( hero ) );
         }
     }
 
-    return msg;
+    return stream;
 }
