@@ -31,7 +31,6 @@
 #include "army_troop.h"
 #include "artifact_info.h"
 #include "castle.h"
-#include "gamedefs.h"
 #include "heroes.h"
 #include "kingdom.h"
 #include "maps.h"
@@ -49,7 +48,7 @@ HeroBase::HeroBase( const int type, const int race )
     : magic_point( 0 )
     , move_point( 0 )
 {
-    bag_artifacts.assign( HEROESMAXARTIFACT, Artifact::UNKNOWN );
+    bag_artifacts.assign( BagArtifacts::maxCapacity, Artifact::UNKNOWN );
     LoadDefaults( type, race );
 }
 
@@ -537,8 +536,8 @@ bool HeroBase::CanCastSpell( const Spell & spell, std::string * res /* = nullptr
         }
 
         if ( spell == Spell::HAUNT || spell == Spell::SETAGUARDIAN || spell == Spell::SETEGUARDIAN || spell == Spell::SETFGUARDIAN || spell == Spell::SETWGUARDIAN ) {
-            const Maps::Tiles & tile = world.GetTiles( hero->GetIndex() );
-            const MP2::MapObjectType object = tile.GetObject( false );
+            const Maps::Tile & tile = world.getTile( hero->GetIndex() );
+            const MP2::MapObjectType object = tile.getMainObjectType( false );
 
             if ( MP2::OBJ_MINE != object ) {
                 if ( res != nullptr ) {
@@ -586,16 +585,14 @@ bool HeroBase::CanLearnSpell( const Spell & spell ) const
              || ( 3 == spell.Level() && Skill::Level::BASIC <= wisdom ) || 3 > spell.Level() );
 }
 
-StreamBase & operator<<( StreamBase & msg, const HeroBase & hero )
+OStreamBase & operator<<( OStreamBase & stream, const HeroBase & hero )
 {
-    return msg << static_cast<const Skill::Primary &>( hero ) << static_cast<const MapPosition &>( hero ) << hero.modes << hero.magic_point << hero.move_point
-               << hero.spell_book << hero.bag_artifacts;
+    return stream << static_cast<const Skill::Primary &>( hero ) << static_cast<const MapPosition &>( hero ) << hero.modes << hero.magic_point << hero.move_point
+                  << hero.spell_book << hero.bag_artifacts;
 }
 
-StreamBase & operator>>( StreamBase & msg, HeroBase & hero )
+IStreamBase & operator>>( IStreamBase & stream, HeroBase & hero )
 {
-    msg >> static_cast<Skill::Primary &>( hero ) >> static_cast<MapPosition &>( hero ) >> hero.modes >> hero.magic_point >> hero.move_point >> hero.spell_book
-        >> hero.bag_artifacts;
-
-    return msg;
+    return stream >> static_cast<Skill::Primary &>( hero ) >> static_cast<MapPosition &>( hero ) >> hero.modes >> hero.magic_point >> hero.move_point >> hero.spell_book
+           >> hero.bag_artifacts;
 }
