@@ -31,7 +31,6 @@
 #include "artifact.h"
 #include "artifact_info.h"
 #include "game_static.h"
-#include "gamedefs.h"
 #include "heroes.h"
 #include "heroes_base.h"
 #include "kingdom.h"
@@ -39,55 +38,56 @@
 #include "rand.h"
 #include "serialize.h"
 #include "skill_static.h"
+#include "spell.h"
 #include "tools.h"
 #include "translations.h"
 #include "world.h"
 
 namespace
 {
-    constexpr std::array<int, MAXSECONDARYSKILL> allSecondarySkills{ Skill::Secondary::PATHFINDING, Skill::Secondary::ARCHERY,    Skill::Secondary::LOGISTICS,
-                                                                     Skill::Secondary::SCOUTING,    Skill::Secondary::DIPLOMACY,  Skill::Secondary::NAVIGATION,
-                                                                     Skill::Secondary::LEADERSHIP,  Skill::Secondary::WISDOM,     Skill::Secondary::MYSTICISM,
-                                                                     Skill::Secondary::LUCK,        Skill::Secondary::BALLISTICS, Skill::Secondary::EAGLE_EYE,
-                                                                     Skill::Secondary::NECROMANCY,  Skill::Secondary::ESTATES };
+    constexpr std::array<int, Skill::numOfSecondarySkills> allSecondarySkills{ Skill::Secondary::PATHFINDING, Skill::Secondary::ARCHERY,    Skill::Secondary::LOGISTICS,
+                                                                               Skill::Secondary::SCOUTING,    Skill::Secondary::DIPLOMACY,  Skill::Secondary::NAVIGATION,
+                                                                               Skill::Secondary::LEADERSHIP,  Skill::Secondary::WISDOM,     Skill::Secondary::MYSTICISM,
+                                                                               Skill::Secondary::LUCK,        Skill::Secondary::BALLISTICS, Skill::Secondary::EAGLE_EYE,
+                                                                               Skill::Secondary::NECROMANCY,  Skill::Secondary::ESTATES };
     static_assert( !allSecondarySkills.empty() && allSecondarySkills.back() != 0, "All existing secondary skills must be present in this array" );
 
     int SecondaryGetWeightSkillFromRace( const int race, const int skill )
     {
-        const Skill::stats_t * ptr = GameStatic::GetSkillStats( race );
+        const Skill::FactionProperties * ptr = GameStatic::GetFactionProperties( race );
         if ( ptr == nullptr ) {
             return 0;
         }
 
         switch ( skill ) {
         case Skill::Secondary::PATHFINDING:
-            return ptr->mature_secondary.pathfinding;
+            return ptr->weightsOfSecondarySkills.pathfinding;
         case Skill::Secondary::ARCHERY:
-            return ptr->mature_secondary.archery;
+            return ptr->weightsOfSecondarySkills.archery;
         case Skill::Secondary::LOGISTICS:
-            return ptr->mature_secondary.logistics;
+            return ptr->weightsOfSecondarySkills.logistics;
         case Skill::Secondary::SCOUTING:
-            return ptr->mature_secondary.scouting;
+            return ptr->weightsOfSecondarySkills.scouting;
         case Skill::Secondary::DIPLOMACY:
-            return ptr->mature_secondary.diplomacy;
+            return ptr->weightsOfSecondarySkills.diplomacy;
         case Skill::Secondary::NAVIGATION:
-            return ptr->mature_secondary.navigation;
+            return ptr->weightsOfSecondarySkills.navigation;
         case Skill::Secondary::LEADERSHIP:
-            return ptr->mature_secondary.leadership;
+            return ptr->weightsOfSecondarySkills.leadership;
         case Skill::Secondary::WISDOM:
-            return ptr->mature_secondary.wisdom;
+            return ptr->weightsOfSecondarySkills.wisdom;
         case Skill::Secondary::MYSTICISM:
-            return ptr->mature_secondary.mysticism;
+            return ptr->weightsOfSecondarySkills.mysticism;
         case Skill::Secondary::LUCK:
-            return ptr->mature_secondary.luck;
+            return ptr->weightsOfSecondarySkills.luck;
         case Skill::Secondary::BALLISTICS:
-            return ptr->mature_secondary.ballistics;
+            return ptr->weightsOfSecondarySkills.ballistics;
         case Skill::Secondary::EAGLE_EYE:
-            return ptr->mature_secondary.eagleeye;
+            return ptr->weightsOfSecondarySkills.eagleeye;
         case Skill::Secondary::NECROMANCY:
-            return ptr->mature_secondary.necromancy;
+            return ptr->weightsOfSecondarySkills.necromancy;
         case Skill::Secondary::ESTATES:
-            return ptr->mature_secondary.estates;
+            return ptr->weightsOfSecondarySkills.estates;
         default:
             assert( 0 );
             break;
@@ -98,7 +98,7 @@ namespace
 
     int SecondaryPriorityFromRace( const int race, const std::unordered_set<int> & blacklist, const uint32_t seed )
     {
-        Rand::Queue parts( MAXSECONDARYSKILL );
+        Rand::Queue parts( Skill::numOfSecondarySkills );
 
         for ( auto skill : allSecondarySkills ) {
             if ( blacklist.find( skill ) != blacklist.end() ) {
@@ -116,75 +116,77 @@ namespace
     }
 }
 
-uint32_t Skill::Secondary::GetValues() const
+uint32_t Skill::Secondary::GetValue() const
 {
-    const values_t * val = GameStatic::GetSkillValues( Skill() );
+    const SecondarySkillValuesPerLevel * ptr = GameStatic::GetSecondarySkillValuesPerLevel( Skill() );
+    if ( ptr == nullptr ) {
+        return 0;
+    }
 
-    if ( val )
-        switch ( Level() ) {
-        case Level::BASIC:
-            return val->values.basic;
-        case Level::ADVANCED:
-            return val->values.advanced;
-        case Level::EXPERT:
-            return val->values.expert;
-        default:
-            break;
-        }
+    switch ( Level() ) {
+    case Level::BASIC:
+        return ptr->values.basic;
+    case Level::ADVANCED:
+        return ptr->values.advanced;
+    case Level::EXPERT:
+        return ptr->values.expert;
+    default:
+        break;
+    }
 
     return 0;
 }
 
-Skill::Primary::Primary()
-    : attack( 0 )
-    , defense( 0 )
-    , power( 0 )
-    , knowledge( 0 )
-{}
-
 void Skill::Primary::LoadDefaults( int type, int race )
 {
-    const stats_t * ptr = GameStatic::GetSkillStats( race );
+    const FactionProperties * ptr = GameStatic::GetFactionProperties( race );
+    if ( ptr == nullptr ) {
+        return;
+    }
 
-    if ( ptr )
-        switch ( type ) {
-        case HeroBase::CAPTAIN:
-            attack = ptr->captain_primary.attack;
-            defense = ptr->captain_primary.defense;
-            power = ptr->captain_primary.power;
-            knowledge = ptr->captain_primary.knowledge;
-            break;
+    switch ( type ) {
+    case HeroBase::CAPTAIN:
+        attack = ptr->captainPrimarySkills.attack;
+        defense = ptr->captainPrimarySkills.defense;
+        power = ptr->captainPrimarySkills.power;
+        knowledge = ptr->captainPrimarySkills.knowledge;
+        break;
 
-        case HeroBase::HEROES:
-            attack = ptr->initial_primary.attack;
-            defense = ptr->initial_primary.defense;
-            power = ptr->initial_primary.power;
-            knowledge = ptr->initial_primary.knowledge;
-            break;
+    case HeroBase::HEROES:
+        attack = ptr->heroInitialPrimarySkills.attack;
+        defense = ptr->heroInitialPrimarySkills.defense;
+        power = ptr->heroInitialPrimarySkills.power;
+        knowledge = ptr->heroInitialPrimarySkills.knowledge;
+        break;
 
-        default:
-            break;
-        }
+    default:
+        break;
+    }
 }
 
 int Skill::Primary::GetInitialSpell( int race )
 {
-    const stats_t * ptr = GameStatic::GetSkillStats( race );
-    return ptr ? ptr->initial_spell : 0;
+    const FactionProperties * ptr = GameStatic::GetFactionProperties( race );
+    if ( ptr == nullptr ) {
+        return Spell::NONE;
+    }
+
+    return ptr->initialSpell;
 }
 
 int Skill::Primary::getHeroDefaultSkillValue( const int skill, const int race )
 {
-    if ( const stats_t * ptr = GameStatic::GetSkillStats( race ) ) {
+    const FactionProperties * ptr = GameStatic::GetFactionProperties( race );
+    if ( ptr ) {
         switch ( skill ) {
         case ATTACK:
-            return ptr->initial_primary.attack;
+            return ptr->heroInitialPrimarySkills.attack;
         case DEFENSE:
-            return ptr->initial_primary.defense;
+            return ptr->heroInitialPrimarySkills.defense;
         case POWER:
-            return ptr->initial_primary.power;
+            return ptr->heroInitialPrimarySkills.power;
         case KNOWLEDGE:
-            return ptr->initial_primary.knowledge;
+            return ptr->heroInitialPrimarySkills.knowledge;
         default:
             // Are you sure that you are passing the correct skill type?
             assert( 0 );
@@ -192,30 +194,32 @@ int Skill::Primary::getHeroDefaultSkillValue( const int skill, const int race )
         }
     }
 
-    return skill == POWER ? 1 : 0;
+    return ( skill == POWER || skill == KNOWLEDGE ) ? 1 : 0;
 }
 
 int Skill::Primary::LevelUp( int race, int level, uint32_t seed )
 {
-    Rand::Queue percents( MAXPRIMARYSKILL );
-
-    const stats_t * ptr = GameStatic::GetSkillStats( race );
-    if ( ptr ) {
-        if ( ptr->over_level > level ) {
-            percents.Push( ATTACK, ptr->mature_primary_under.attack );
-            percents.Push( DEFENSE, ptr->mature_primary_under.defense );
-            percents.Push( POWER, ptr->mature_primary_under.power );
-            percents.Push( KNOWLEDGE, ptr->mature_primary_under.knowledge );
-        }
-        else {
-            percents.Push( ATTACK, ptr->mature_primary_over.attack );
-            percents.Push( DEFENSE, ptr->mature_primary_over.defense );
-            percents.Push( POWER, ptr->mature_primary_over.power );
-            percents.Push( KNOWLEDGE, ptr->mature_primary_over.knowledge );
-        }
+    const FactionProperties * ptr = GameStatic::GetFactionProperties( race );
+    if ( ptr == nullptr ) {
+        return UNKNOWN;
     }
 
-    int result = percents.Size() ? percents.GetWithSeed( seed ) : UNKNOWN;
+    Rand::Queue percents( numOfPrimarySkills );
+
+    if ( ptr->boundaryBetweenLowAndHighLevels > level ) {
+        percents.Push( ATTACK, ptr->weightsOfPrimarySkillsForLowLevels.attack );
+        percents.Push( DEFENSE, ptr->weightsOfPrimarySkillsForLowLevels.defense );
+        percents.Push( POWER, ptr->weightsOfPrimarySkillsForLowLevels.power );
+        percents.Push( KNOWLEDGE, ptr->weightsOfPrimarySkillsForLowLevels.knowledge );
+    }
+    else {
+        percents.Push( ATTACK, ptr->weightsOfPrimarySkillsForHighLevels.attack );
+        percents.Push( DEFENSE, ptr->weightsOfPrimarySkillsForHighLevels.defense );
+        percents.Push( POWER, ptr->weightsOfPrimarySkillsForHighLevels.power );
+        percents.Push( KNOWLEDGE, ptr->weightsOfPrimarySkillsForHighLevels.knowledge );
+    }
+
+    const int result = percents.GetWithSeed( seed );
 
     switch ( result ) {
     case ATTACK:
@@ -231,6 +235,7 @@ int Skill::Primary::LevelUp( int race, int level, uint32_t seed )
         ++knowledge;
         break;
     default:
+        assert( 0 );
         break;
     }
 
@@ -382,47 +387,6 @@ bool Skill::Secondary::isValid() const
     return Skill() != UNKNOWN && Level() != Level::NONE;
 }
 
-int Skill::Secondary::RandForWitchsHut()
-{
-    const Skill::secondary_t * sec = GameStatic::GetSkillForWitchsHut();
-    std::vector<int> v;
-
-    if ( sec ) {
-        v.reserve( 14 );
-
-        if ( sec->archery )
-            v.push_back( ARCHERY );
-        if ( sec->ballistics )
-            v.push_back( BALLISTICS );
-        if ( sec->diplomacy )
-            v.push_back( DIPLOMACY );
-        if ( sec->eagleeye )
-            v.push_back( EAGLE_EYE );
-        if ( sec->estates )
-            v.push_back( ESTATES );
-        if ( sec->leadership )
-            v.push_back( LEADERSHIP );
-        if ( sec->logistics )
-            v.push_back( LOGISTICS );
-        if ( sec->luck )
-            v.push_back( LUCK );
-        if ( sec->mysticism )
-            v.push_back( MYSTICISM );
-        if ( sec->navigation )
-            v.push_back( NAVIGATION );
-        if ( sec->necromancy )
-            v.push_back( NECROMANCY );
-        if ( sec->pathfinding )
-            v.push_back( PATHFINDING );
-        if ( sec->scouting )
-            v.push_back( SCOUTING );
-        if ( sec->wisdom )
-            v.push_back( WISDOM );
-    }
-
-    return v.empty() ? UNKNOWN : Rand::Get( v );
-}
-
 int Skill::Secondary::GetIndexSprite1() const
 {
     return ( Skill() > UNKNOWN && Skill() <= ESTATES ) ? Skill() : 0;
@@ -499,7 +463,7 @@ std::string Skill::Secondary::GetNameWithBonus( const Heroes & hero ) const
 
 std::string Skill::Secondary::GetDescription( const Heroes & hero ) const
 {
-    uint32_t count = GetValues();
+    uint32_t count = GetValue();
     std::string name = GetName();
     std::string str = "unknown";
 
@@ -616,7 +580,7 @@ std::string Skill::Secondary::GetDescription( const Heroes & hero ) const
         break;
     }
     case NECROMANCY: {
-        count += Skill::GetNecromancyPercent( hero ) - hero.GetSecondaryValues( Skill::Secondary::NECROMANCY );
+        count += Skill::GetNecromancyPercent( hero ) - hero.GetSecondarySkillValue( Skill::Secondary::NECROMANCY );
         name = GetNameWithBonus( hero );
         str = _( "%{skill} allows %{count} percent of the creatures killed in combat to be brought back from the dead as Skeletons." );
         break;
@@ -639,61 +603,84 @@ std::string Skill::Secondary::GetDescription( const Heroes & hero ) const
 
 Skill::SecSkills::SecSkills()
 {
-    reserve( HEROESMAXSKILL );
+    reserve( Heroes::maxNumOfSecSkills );
 }
 
-Skill::SecSkills::SecSkills( int race )
+Skill::SecSkills::SecSkills( const int race )
 {
-    reserve( HEROESMAXSKILL );
+    reserve( Heroes::maxNumOfSecSkills );
 
-    if ( race & Race::ALL ) {
-        const stats_t * ptr = GameStatic::GetSkillStats( race );
+    if ( !( race & Race::ALL ) ) {
+        return;
+    }
 
-        if ( ptr ) {
-            if ( ptr->initial_secondary.archery )
-                AddSkill( Secondary( Secondary::ARCHERY, ptr->initial_secondary.archery ) );
-            if ( ptr->initial_secondary.diplomacy )
-                AddSkill( Secondary( Secondary::DIPLOMACY, ptr->initial_secondary.diplomacy ) );
-            if ( ptr->initial_secondary.eagleeye )
-                AddSkill( Secondary( Secondary::EAGLE_EYE, ptr->initial_secondary.eagleeye ) );
-            if ( ptr->initial_secondary.estates )
-                AddSkill( Secondary( Secondary::ESTATES, ptr->initial_secondary.estates ) );
-            if ( ptr->initial_secondary.logistics )
-                AddSkill( Secondary( Secondary::LOGISTICS, ptr->initial_secondary.logistics ) );
-            if ( ptr->initial_secondary.luck )
-                AddSkill( Secondary( Secondary::LUCK, ptr->initial_secondary.luck ) );
-            if ( ptr->initial_secondary.mysticism )
-                AddSkill( Secondary( Secondary::MYSTICISM, ptr->initial_secondary.mysticism ) );
-            if ( ptr->initial_secondary.pathfinding )
-                AddSkill( Secondary( Secondary::PATHFINDING, ptr->initial_secondary.pathfinding ) );
-            if ( ptr->initial_secondary.leadership )
-                AddSkill( Secondary( Secondary::LEADERSHIP, ptr->initial_secondary.leadership ) );
-            if ( ptr->initial_secondary.ballistics )
-                AddSkill( Secondary( Secondary::BALLISTICS, ptr->initial_secondary.ballistics ) );
-            if ( ptr->initial_secondary.navigation )
-                AddSkill( Secondary( Secondary::NAVIGATION, ptr->initial_secondary.navigation ) );
-            if ( ptr->initial_secondary.scouting )
-                AddSkill( Secondary( Secondary::SCOUTING, ptr->initial_secondary.scouting ) );
-            if ( ptr->initial_secondary.necromancy )
-                AddSkill( Secondary( Secondary::NECROMANCY, ptr->initial_secondary.necromancy ) );
-            if ( ptr->initial_secondary.wisdom )
-                AddSkill( Secondary( Secondary::WISDOM, ptr->initial_secondary.wisdom ) );
-        }
+    const FactionProperties * ptr = GameStatic::GetFactionProperties( race );
+    if ( ptr == nullptr ) {
+        return;
+    }
+
+    if ( ptr->heroInitialSecondarySkills.archery ) {
+        AddSkill( Secondary( Secondary::ARCHERY, ptr->heroInitialSecondarySkills.archery ) );
+    }
+    if ( ptr->heroInitialSecondarySkills.diplomacy ) {
+        AddSkill( Secondary( Secondary::DIPLOMACY, ptr->heroInitialSecondarySkills.diplomacy ) );
+    }
+    if ( ptr->heroInitialSecondarySkills.eagleeye ) {
+        AddSkill( Secondary( Secondary::EAGLE_EYE, ptr->heroInitialSecondarySkills.eagleeye ) );
+    }
+    if ( ptr->heroInitialSecondarySkills.estates ) {
+        AddSkill( Secondary( Secondary::ESTATES, ptr->heroInitialSecondarySkills.estates ) );
+    }
+    if ( ptr->heroInitialSecondarySkills.logistics ) {
+        AddSkill( Secondary( Secondary::LOGISTICS, ptr->heroInitialSecondarySkills.logistics ) );
+    }
+    if ( ptr->heroInitialSecondarySkills.luck ) {
+        AddSkill( Secondary( Secondary::LUCK, ptr->heroInitialSecondarySkills.luck ) );
+    }
+    if ( ptr->heroInitialSecondarySkills.mysticism ) {
+        AddSkill( Secondary( Secondary::MYSTICISM, ptr->heroInitialSecondarySkills.mysticism ) );
+    }
+    if ( ptr->heroInitialSecondarySkills.pathfinding ) {
+        AddSkill( Secondary( Secondary::PATHFINDING, ptr->heroInitialSecondarySkills.pathfinding ) );
+    }
+    if ( ptr->heroInitialSecondarySkills.leadership ) {
+        AddSkill( Secondary( Secondary::LEADERSHIP, ptr->heroInitialSecondarySkills.leadership ) );
+    }
+    if ( ptr->heroInitialSecondarySkills.ballistics ) {
+        AddSkill( Secondary( Secondary::BALLISTICS, ptr->heroInitialSecondarySkills.ballistics ) );
+    }
+    if ( ptr->heroInitialSecondarySkills.navigation ) {
+        AddSkill( Secondary( Secondary::NAVIGATION, ptr->heroInitialSecondarySkills.navigation ) );
+    }
+    if ( ptr->heroInitialSecondarySkills.scouting ) {
+        AddSkill( Secondary( Secondary::SCOUTING, ptr->heroInitialSecondarySkills.scouting ) );
+    }
+    if ( ptr->heroInitialSecondarySkills.necromancy ) {
+        AddSkill( Secondary( Secondary::NECROMANCY, ptr->heroInitialSecondarySkills.necromancy ) );
+    }
+    if ( ptr->heroInitialSecondarySkills.wisdom ) {
+        AddSkill( Secondary( Secondary::WISDOM, ptr->heroInitialSecondarySkills.wisdom ) );
     }
 }
 
 int Skill::SecSkills::GetLevel( int skill ) const
 {
-    const_iterator it = std::find_if( begin(), end(), [skill]( const Secondary & v ) { return v.isSkill( skill ); } );
+    const const_iterator iter = std::find_if( begin(), end(), [skill]( const Secondary & v ) { return v.isSkill( skill ); } );
+    if ( iter == end() ) {
+        return Level::NONE;
+    }
 
-    return it == end() ? Level::NONE : ( *it ).Level();
+    return iter->Level();
 }
 
-uint32_t Skill::SecSkills::GetValues( int skill ) const
+uint32_t Skill::SecSkills::GetValue( int skill ) const
 {
-    const_iterator it = std::find_if( begin(), end(), [skill]( const Secondary & v ) { return v.isSkill( skill ); } );
+    const const_iterator iter = std::find_if( begin(), end(), [skill]( const Secondary & v ) { return v.isSkill( skill ); } );
+    if ( iter == end() ) {
+        return 0;
+    }
 
-    return it == end() ? 0 : ( *it ).GetValues();
+    return iter->GetValue();
 }
 
 int Skill::SecSkills::Count() const
@@ -714,18 +701,27 @@ int Skill::SecSkills::GetTotalLevel() const
 
 void Skill::SecSkills::AddSkill( const Skill::Secondary & skill )
 {
-    if ( skill.isValid() ) {
-        const int skillValue = skill.Skill();
-        iterator it = std::find_if( begin(), end(), [skillValue]( const Secondary & v ) { return v.isSkill( skillValue ); } );
-        if ( it != end() )
-            ( *it ).SetLevel( skill.Level() );
-        else {
-            it = std::find_if( begin(), end(), []( const Secondary & v ) { return !v.isValid(); } );
-            if ( it != end() )
-                ( *it ).Set( skill );
-            else if ( size() < HEROESMAXSKILL )
-                push_back( skill );
-        }
+    if ( !skill.isValid() ) {
+        return;
+    }
+
+    // If there is already such a skill, then just update its level
+    if ( const auto iter = std::find_if( begin(), end(), [skillValue = skill.Skill()]( const Secondary & v ) { return v.isSkill( skillValue ); } ); iter != end() ) {
+        iter->SetLevel( skill.Level() );
+
+        return;
+    }
+
+    // If there is an invalid skill, then replace it with the given skill
+    if ( const auto iter = std::find_if( begin(), end(), []( const Secondary & v ) { return !v.isValid(); } ); iter != end() ) {
+        iter->Set( skill );
+
+        return;
+    }
+
+    // If there is room for a new skill, then add the given skill
+    if ( size() < Heroes::maxNumOfSecSkills ) {
+        push_back( skill );
     }
 }
 
@@ -761,15 +757,16 @@ std::string Skill::SecSkills::String() const
 
 void Skill::SecSkills::FillMax( const Skill::Secondary & skill )
 {
-    if ( size() < HEROESMAXSKILL )
-        resize( HEROESMAXSKILL, skill );
+    if ( size() < Heroes::maxNumOfSecSkills ) {
+        resize( Heroes::maxNumOfSecSkills, skill );
+    }
 }
 
 std::pair<Skill::Secondary, Skill::Secondary> Skill::SecSkills::FindSkillsForLevelUp( const int race, const uint32_t firstSkillSeed,
                                                                                       uint32_t const secondSkillSeed ) const
 {
     std::unordered_set<int> blacklist;
-    blacklist.reserve( MAXSECONDARYSKILL + HEROESMAXSKILL );
+    blacklist.reserve( numOfSecondarySkills + Heroes::maxNumOfSecSkills );
 
     for ( const Secondary & skill : *this ) {
         if ( skill.Level() != Level::EXPERT ) {
@@ -779,7 +776,7 @@ std::pair<Skill::Secondary, Skill::Secondary> Skill::SecSkills::FindSkillsForLev
         blacklist.insert( skill.Skill() );
     }
 
-    if ( Count() >= HEROESMAXSKILL ) {
+    if ( Count() >= Heroes::maxNumOfSecSkills ) {
         for ( const int skill : allSecondarySkills ) {
             if ( GetLevel( skill ) != Level::NONE ) {
                 continue;
@@ -835,40 +832,30 @@ std::pair<Skill::Secondary, Skill::Secondary> Skill::SecSkills::FindSkillsForLev
     return result;
 }
 
-void StringAppendModifiers( std::string & str, int value )
-{
-    if ( value < 0 )
-        str.append( " " ); // '-' present
-    else if ( value > 0 )
-        str.append( " +" );
-
-    str.append( std::to_string( value ) );
-}
-
 int Skill::GetLeadershipModifiers( int level, std::string * strs = nullptr )
 {
     Secondary skill( Secondary::LEADERSHIP, level );
 
-    if ( skill.GetValues() && strs ) {
+    if ( skill.GetValue() && strs ) {
         strs->append( skill.GetName() );
-        StringAppendModifiers( *strs, skill.GetValues() );
+        fheroes2::appendModifierToString( *strs, skill.GetValue() );
         strs->append( "\n" );
     }
 
-    return skill.GetValues();
+    return skill.GetValue();
 }
 
 int Skill::GetLuckModifiers( int level, std::string * strs = nullptr )
 {
     Secondary skill( Secondary::LUCK, level );
 
-    if ( skill.GetValues() && strs ) {
+    if ( skill.GetValue() && strs ) {
         strs->append( skill.GetName() );
-        StringAppendModifiers( *strs, skill.GetValues() );
+        fheroes2::appendModifierToString( *strs, skill.GetValue() );
         strs->append( "\n" );
     }
 
-    return skill.GetValues();
+    return skill.GetValue();
 }
 
 uint32_t Skill::GetNecromancyBonus( const HeroBase & hero )
@@ -881,7 +868,7 @@ uint32_t Skill::GetNecromancyBonus( const HeroBase & hero )
 
 uint32_t Skill::GetNecromancyPercent( const HeroBase & hero )
 {
-    uint32_t percent = hero.GetSecondaryValues( Skill::Secondary::NECROMANCY );
+    uint32_t percent = hero.GetSecondarySkillValue( Skill::Secondary::NECROMANCY );
     percent += 10 * GetNecromancyBonus( hero );
     // cap at 100% bonus
     return std::min( percent, 100u );
@@ -901,33 +888,30 @@ uint32_t Skill::GetDiplomacySurrenderCostDiscount( const int level )
     }
 }
 
-StreamBase & Skill::operator<<( StreamBase & msg, const Primary & skill )
+OStreamBase & Skill::operator<<( OStreamBase & stream, const Primary & skill )
 {
-    return msg << skill.attack << skill.defense << skill.knowledge << skill.power;
+    return stream << skill.attack << skill.defense << skill.knowledge << skill.power;
 }
 
-StreamBase & Skill::operator>>( StreamBase & msg, Primary & skill )
+IStreamBase & Skill::operator>>( IStreamBase & stream, Primary & skill )
 {
-    return msg >> skill.attack >> skill.defense >> skill.knowledge >> skill.power;
+    return stream >> skill.attack >> skill.defense >> skill.knowledge >> skill.power;
 }
 
-StreamBase & Skill::operator>>( StreamBase & sb, Secondary & st )
-{
-    return sb >> st.first >> st.second;
-}
-
-StreamBase & Skill::operator<<( StreamBase & sb, const SecSkills & ss )
+OStreamBase & Skill::operator<<( OStreamBase & stream, const SecSkills & ss )
 {
     const std::vector<Secondary> & v = ss;
-    return sb << v;
+    return stream << v;
 }
 
-StreamBase & Skill::operator>>( StreamBase & sb, SecSkills & ss )
+IStreamBase & Skill::operator>>( IStreamBase & stream, SecSkills & ss )
 {
     std::vector<Secondary> & v = ss;
-    sb >> v;
+    stream >> v;
 
-    if ( v.size() > HEROESMAXSKILL )
-        v.resize( HEROESMAXSKILL );
-    return sb;
+    if ( v.size() > Heroes::maxNumOfSecSkills ) {
+        v.resize( Heroes::maxNumOfSecSkills );
+    }
+
+    return stream;
 }
