@@ -1,6 +1,6 @@
 /***************************************************************************
  *   fheroes2: https://github.com/ihhub/fheroes2                           *
- *   Copyright (C) 2019 - 2023                                             *
+ *   Copyright (C) 2019 - 2024                                             *
  *                                                                         *
  *   Free Heroes2 Engine: http://sourceforge.net/projects/fheroes2         *
  *   Copyright (C) 2009 by Andrey Afletdinov <fheroes2@gmail.com>          *
@@ -20,10 +20,11 @@
  *   Free Software Foundation, Inc.,                                       *
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
  ***************************************************************************/
-#ifndef H2KINGDOM_H
-#define H2KINGDOM_H
+
+#pragma once
 
 #include <algorithm>
+#include <array>
 #include <cstdint>
 #include <functional>
 #include <list>
@@ -31,23 +32,27 @@
 
 #include "bitmodes.h"
 #include "castle.h"
-#include "gamedefs.h"
 #include "heroes.h"
 #include "heroes_recruits.h"
 #include "monster.h"
-#include "mp2.h"
 #include "pairs.h"
 #include "players.h"
 #include "puzzle.h"
 #include "resource.h"
 
-class StreamBase;
+class IStreamBase;
+class OStreamBase;
 
 struct EventDate;
 
+namespace MP2
+{
+    enum MapObjectType : uint16_t;
+}
+
 namespace Maps
 {
-    class Tiles;
+    class Tile;
 }
 
 class Kingdom : public BitModes, public Control
@@ -55,7 +60,7 @@ class Kingdom : public BitModes, public Control
 public:
     enum
     {
-        // UNDEF      = 0x0001,
+        // UNUSED = 0x0001,
         IDENTIFYHERO = 0x0002,
         // UNUSED = 0x0004,
         KINGDOM_OVERVIEW_CASTLE_SELECTION = 0x0008
@@ -72,9 +77,16 @@ public:
     };
 
     Kingdom();
+
+    Kingdom( const Kingdom & ) = delete;
+    Kingdom( Kingdom && ) = default;
+
     ~Kingdom() override = default;
 
-    void Init( int color );
+    Kingdom & operator=( const Kingdom & ) = delete;
+    Kingdom & operator=( Kingdom && ) = default;
+
+    void Init( const int clr );
     void clear();
 
     void openOverviewDialog();
@@ -84,23 +96,20 @@ public:
     bool AllowPayment( const Funds & ) const;
     bool AllowRecruitHero( bool check_payment ) const;
 
-    // Return true if this kingdom can recruit heroes, false otherwise. For
-    // example this function will return false when kingdom has one town that
-    // cannot be upgraded to a castle.
+    // Returns true if this kingdom can recruit heroes, false otherwise. For example, this function returns false if there is only one town in the kingdom that cannot be
+    // upgraded to a castle.
     bool canRecruitHeroes() const
     {
-        return std::any_of( castles.begin(), castles.end(), []( const Castle * castle ) { return ( castle->isCastle() || castle->Modes( Castle::ALLOWCASTLE ) ); } );
+        return std::any_of( castles.begin(), castles.end(),
+                            []( const Castle * castle ) { return ( castle->isCastle() || !castle->isBuildingDisabled( BUILD_CASTLE ) ); } );
     }
 
-    // Return true if this kingdom has any heroes, false otherwise.
+    // Returns true if this kingdom has any heroes, false otherwise.
     bool hasHeroes() const
     {
         return !heroes.empty();
     }
 
-    void SetLastBattleWinHero( const Heroes & hero );
-    Heroes * GetLastBattleWinHero() const;
-
     void appendSurrenderedHero( Heroes & hero );
 
     Heroes * GetBestHero() const;
@@ -115,6 +124,7 @@ public:
     {
         return resource;
     }
+
     Funds GetIncome( int type = INCOME_ALL ) const;
 
     double GetArmiesStrength() const;
@@ -166,7 +176,7 @@ public:
     void RemoveHero( const Heroes * hero );
     void ApplyPlayWithStartingHero();
 
-    void AddCastle( const Castle * );
+    void AddCastle( Castle * castle );
     void RemoveCastle( const Castle * );
 
     void ActionBeforeTurn();
@@ -178,10 +188,10 @@ public:
     void SetVisited( int32_t index, const MP2::MapObjectType objectType );
     uint32_t CountVisitedObjects( const MP2::MapObjectType objectType ) const;
     bool isVisited( const MP2::MapObjectType objectType ) const;
-    bool isVisited( const Maps::Tiles & ) const;
+    bool isVisited( const Maps::Tile & ) const;
     bool isVisited( int32_t, const MP2::MapObjectType objectType ) const;
 
-    bool isValidKingdomObject( const Maps::Tiles & tile, const MP2::MapObjectType objectType ) const;
+    bool isValidKingdomObject( const Maps::Tile & tile, const MP2::MapObjectType objectType ) const;
 
     bool opponentsCanRecruitMoreHeroes() const;
     bool opponentsHaveHeroes() const;
@@ -202,13 +212,12 @@ public:
     static uint32_t GetMaxHeroes();
 
 private:
-    cost_t _getKingdomStartingResources( const int difficulty ) const;
+    Cost _getKingdomStartingResources( const int difficulty ) const;
 
-    friend StreamBase & operator<<( StreamBase &, const Kingdom & );
-    friend StreamBase & operator>>( StreamBase &, Kingdom & );
+    friend OStreamBase & operator<<( OStreamBase & stream, const Kingdom & kingdom );
+    friend IStreamBase & operator>>( IStreamBase & stream, Kingdom & kingdom );
 
     int color;
-    int _lastBattleWinHeroID;
     Funds resource;
 
     uint32_t lost_town_days;
@@ -242,31 +251,22 @@ public:
     void NewWeek();
     void NewMonth();
 
-    Kingdom & GetKingdom( int color );
-    const Kingdom & GetKingdom( int color ) const;
+    Kingdom & GetKingdom( const int color );
+    const Kingdom & GetKingdom( const int color ) const;
 
     int GetNotLossColors() const;
-    int FindWins( int ) const;
+    int FindWins( const int cond ) const;
 
-    void AddHeroes( const AllHeroes & );
-    void AddCastles( const AllCastles & );
+    void AddHeroes( const AllHeroes & heroes );
+    void AddCastles( const AllCastles & castles );
 
     // Resets recruits in all kingdoms and returns a set of heroes that are still available for recruitment
     // in the kingdoms
     std::set<Heroes *> resetRecruits();
 
 private:
-    friend StreamBase & operator<<( StreamBase &, const Kingdoms & );
-    friend StreamBase & operator>>( StreamBase &, Kingdoms & );
+    friend OStreamBase & operator<<( OStreamBase & stream, const Kingdoms & obj );
+    friend IStreamBase & operator>>( IStreamBase & stream, Kingdoms & obj );
 
-    static constexpr uint32_t _size = KINGDOMMAX + 1;
-    Kingdom kingdoms[_size];
+    std::array<Kingdom, maxNumOfPlayers + 1> _kingdoms;
 };
-
-StreamBase & operator<<( StreamBase &, const Kingdom & );
-StreamBase & operator>>( StreamBase &, Kingdom & );
-
-StreamBase & operator<<( StreamBase &, const Kingdoms & );
-StreamBase & operator>>( StreamBase &, Kingdoms & );
-
-#endif
