@@ -25,9 +25,9 @@
 #include <cassert>
 #include <cstddef>
 #include <cstdint>
+#include <initializer_list>
 #include <list>
 #include <map>
-#include <memory>
 #include <set>
 #include <utility>
 
@@ -35,15 +35,20 @@
 #include "army_troop.h"
 #include "castle.h"
 #include "color.h"
-#include "gamedefs.h"
+#include "direction.h"
+#include "ground.h"
 #include "heroes.h"
 #include "map_format_info.h"
 #include "map_object_info.h"
+#include "maps.h"
 #include "maps_fileinfo.h"
 #include "maps_tiles.h"
 #include "maps_tiles_helper.h"
+#include "monster.h"
 #include "mp2.h"
+#include "players.h"
 #include "race.h"
+#include "rand.h"
 #include "world.h"
 #include "world_object_uid.h"
 
@@ -116,6 +121,168 @@ namespace
             }
         }
     }
+
+    // This function only checks for Streams and ignores River Deltas.
+    bool isStreamPresent( const Maps::Map_Format::TileInfo & mapTile )
+    {
+        return std::any_of( mapTile.objects.cbegin(), mapTile.objects.cend(),
+                            []( const Maps::Map_Format::TileObjectInfo & object ) { return object.group == Maps::ObjectGroup::STREAMS; } );
+    }
+
+    bool isStreamToDeltaConnectionNeeded( Maps::Map_Format::MapFormat & map, const int32_t tileId, const int direction )
+    {
+        if ( direction != Direction::TOP && direction != Direction::BOTTOM && direction != Direction::RIGHT && direction != Direction::LEFT ) {
+            return false;
+        }
+
+        if ( !Maps::isValidDirection( tileId, direction ) ) {
+            return false;
+        }
+
+        // The center tile of river delta is located in the next tile.
+        const int32_t nextTileId = Maps::GetDirectionIndex( tileId, direction );
+
+        return std::any_of( map.tiles[nextTileId].objects.cbegin(), map.tiles[nextTileId].objects.cend(),
+                            [&direction]( const Maps::Map_Format::TileObjectInfo & object ) {
+                                return Maps::getRiverDeltaDirectionByIndex( object.group, static_cast<int32_t>( object.index ) ) == Direction::Reflect( direction );
+                            } );
+    }
+
+    // Returns the direction vector bits from 'centerTileIndex' to the around tiles with streams.
+    int getStreamDirecton( Maps::Map_Format::MapFormat & map, const int32_t currentTileId, const bool forceStreamOnTile )
+    {
+        assert( currentTileId >= 0 && map.tiles.size() > static_cast<size_t>( currentTileId ) );
+
+        // Stream includes also the Deltas. For the current tile we need to check only streams excluding Deltas.
+        if ( !forceStreamOnTile && !isStreamPresent( map.tiles[currentTileId] ) ) {
+            // Current tile has no streams.
+            return Direction::UNKNOWN;
+        }
+
+        int streamDirection = Direction::CENTER;
+
+        // For streams we can check only the next four directions.
+        for ( const int direction : { Direction::LEFT, Direction::TOP, Direction::RIGHT, Direction::BOTTOM } ) {
+            if ( !Maps::isValidDirection( currentTileId, direction ) ) {
+                continue;
+            }
+
+            const int32_t tileId = Maps::GetDirectionIndex( currentTileId, direction );
+            assert( tileId >= 0 && map.tiles.size() > static_cast<size_t>( tileId ) );
+
+            // Check also for Deltas connection.
+            if ( isStreamPresent( map.tiles[tileId] ) || isStreamToDeltaConnectionNeeded( map, tileId, direction ) ) {
+                streamDirection |= direction;
+            }
+        }
+
+        return streamDirection;
+    }
+
+    bool hasBits( const int value, const int bits )
+    {
+        return ( value & bits ) == bits;
+    }
+
+    bool hasNoBits( const int value, const int bits )
+    {
+        return ( value & bits ) == 0;
+    }
+
+    uint8_t getStreamIndex( const int streamDirection )
+    {
+        if ( hasNoBits( streamDirection, Direction::CENTER ) ) {
+            // This tile should not have a stream image.
+            assert( 0 );
+            return 255U;
+        }
+
+        if ( hasBits( streamDirection, Direction::LEFT | Direction::BOTTOM ) && hasNoBits( streamDirection, Direction::TOP | Direction::RIGHT ) ) {
+            // \ - stream from the left to the bottom.
+            return 0U;
+        }
+        if ( hasBits( streamDirection, Direction::RIGHT | Direction::BOTTOM ) && hasNoBits( streamDirection, Direction::TOP | Direction::LEFT ) ) {
+            // / - stream from the right to the bottom.
+            return 1U;
+        }
+        if ( hasBits( streamDirection, Direction::RIGHT | Direction::TOP ) && hasNoBits( streamDirection, Direction::BOTTOM | Direction::LEFT ) ) {
+            // \ - stream from the top to the right.
+            return 4U;
+        }
+        if ( hasBits( streamDirection, Direction::LEFT | Direction::TOP ) && hasNoBits( streamDirection, Direction::BOTTOM | Direction::RIGHT ) ) {
+            // / - stream from the top to the left.
+            return 7U;
+        }
+        if ( hasBits( streamDirection, Direction::LEFT | Direction::TOP | Direction::RIGHT ) && hasNoBits( streamDirection, Direction::BOTTOM ) ) {
+            // _|_ - stream from the top to the left and right.
+            return 8U;
+        }
+        if ( hasBits( streamDirection, Direction::BOTTOM | Direction::TOP | Direction::RIGHT ) && hasNoBits( streamDirection, Direction::LEFT ) ) {
+            // |- - stream from the top to the right and bottom.
+            return 9U;
+        }
+        if ( hasBits( streamDirection, Direction::BOTTOM | Direction::TOP | Direction::LEFT ) && hasNoBits( streamDirection, Direction::RIGHT ) ) {
+            // -| - stream from the top to the left and bottom.
+            return 10U;
+        }
+        if ( hasBits( streamDirection, Direction::BOTTOM | Direction::LEFT | Direction::RIGHT ) && hasNoBits( streamDirection, Direction::TOP ) ) {
+            // \/ - stream from the left and right to the bottom.
+            return 11U;
+        }
+        if ( hasBits( streamDirection, Direction::BOTTOM | Direction::LEFT | Direction::TOP | Direction::RIGHT ) ) {
+            // -|- - streams are all around.
+            return 6U;
+        }
+        if ( ( hasBits( streamDirection, Direction::LEFT ) || hasBits( streamDirection, Direction::RIGHT ) )
+             && hasNoBits( streamDirection, Direction::TOP | Direction::BOTTOM ) ) {
+            // - - horizontal stream.
+            return Rand::Get( 1 ) ? 2U : 5U;
+        }
+
+        // | - in all other cases are the vertical stream sprite, including the case when there are no other streams around.
+        return Rand::Get( 1 ) ? 3U : 12U;
+    }
+
+    void updateStreamObjectOnMapTile( Maps::Map_Format::MapFormat & map, const int32_t tileId, const bool forceStreamOnTile )
+    {
+        const int direction = getStreamDirecton( map, tileId, forceStreamOnTile );
+
+        if ( hasNoBits( direction, Direction::CENTER ) ) {
+            // There is no stream on this tile.
+            return;
+        }
+
+        const uint8_t objectIndex = getStreamIndex( direction );
+
+        if ( objectIndex == 255U ) {
+            return;
+        }
+
+        assert( tileId >= 0 && map.tiles.size() > static_cast<size_t>( tileId ) );
+
+        auto & tileObjects = map.tiles[tileId].objects;
+
+        auto streamIter = std::find_if( tileObjects.begin(), tileObjects.end(),
+                                        []( const Maps::Map_Format::TileObjectInfo & object ) { return object.group == Maps::ObjectGroup::STREAMS; } );
+
+        if ( streamIter == tileObjects.end() ) {
+            // Add stream to this tile.
+            const auto & objectInfo = Maps::getObjectInfo( Maps::ObjectGroup::STREAMS, objectIndex );
+
+            if ( !Maps::setObjectOnTile( world.getTile( tileId ), objectInfo, true ) ) {
+                assert( 0 );
+                return;
+            }
+
+            Maps::addObjectToMap( map, tileId, Maps::ObjectGroup::STREAMS, objectIndex );
+        }
+        else {
+            streamIter->index = objectIndex;
+
+            // Update image index for the `world` tile.
+            Maps::Tile::updateTileObjectIcnIndex( world.getTile( tileId ), streamIter->id, objectIndex );
+        }
+    }
 }
 
 namespace Maps
@@ -141,14 +308,14 @@ namespace Maps
 
         // We must clear all tiles before writing something on them.
         for ( size_t i = 0; i < map.tiles.size(); ++i ) {
-            auto & tile = world.GetTiles( static_cast<int32_t>( i ) );
+            auto & tile = world.getTile( static_cast<int32_t>( i ) );
             tile = {};
 
             tile.setIndex( static_cast<int32_t>( i ) );
         }
 
         for ( size_t i = 0; i < map.tiles.size(); ++i ) {
-            readTileTerrain( world.GetTiles( static_cast<int32_t>( i ) ), map.tiles[i] );
+            readTileTerrain( world.getTile( static_cast<int32_t>( i ) ), map.tiles[i] );
         }
 
         // Read objects from all tiles and place them based on their IDs.
@@ -166,7 +333,7 @@ namespace Maps
 
         for ( const auto & info : sortedObjects ) {
             assert( info.info != nullptr );
-            if ( !readTileObject( world.GetTiles( info.tileIndex ), *info.info ) ) {
+            if ( !readTileObject( world.getTile( info.tileIndex ), *info.info ) ) {
                 return false;
             }
         }
@@ -185,18 +352,18 @@ namespace Maps
         map.tiles.resize( size );
 
         for ( size_t i = 0; i < size; ++i ) {
-            writeTile( world.GetTiles( static_cast<int32_t>( i ) ), map.tiles[i] );
+            writeTile( world.getTile( static_cast<int32_t>( i ) ), map.tiles[i] );
         }
 
         return true;
     }
 
-    void readTileTerrain( Tiles & tile, const Map_Format::TileInfo & info )
+    void readTileTerrain( Tile & tile, const Map_Format::TileInfo & info )
     {
         tile.setTerrain( info.terrainIndex, info.terrainFlag & 2, info.terrainFlag & 1 );
     }
 
-    bool readTileObject( Tiles & tile, const Map_Format::TileObjectInfo & object )
+    bool readTileObject( Tile & tile, const Map_Format::TileObjectInfo & object )
     {
         const auto & objectInfos = getObjectsByGroup( object.group );
         if ( object.index >= objectInfos.size() ) {
@@ -218,26 +385,19 @@ namespace Maps
         return setObjectOnTile( tile, objectInfos[object.index], false );
     }
 
-    void writeTile( const Tiles & tile, Map_Format::TileInfo & info )
+    void writeTile( const Tile & tile, Map_Format::TileInfo & info )
     {
         // A tile cannot contain an exactly the same road or stream parts.
         std::set<std::pair<uint32_t, uint8_t>> roadParts;
-        std::set<std::pair<uint32_t, uint8_t>> streamParts;
 
-        const MP2::ObjectIcnType mainObjectIcnType = tile.getObjectIcnType();
+        const MP2::ObjectIcnType mainObjectIcnType = tile.getMainObjectPart().icnType;
         if ( mainObjectIcnType == MP2::OBJ_ICN_TYPE_ROAD ) {
-            roadParts.emplace( tile.GetObjectUID(), tile.GetObjectSpriteIndex() );
-        }
-        else if ( mainObjectIcnType == MP2::OBJ_ICN_TYPE_STREAM ) {
-            streamParts.emplace( tile.GetObjectUID(), tile.GetObjectSpriteIndex() );
+            roadParts.emplace( tile.getMainObjectPart()._uid, tile.getMainObjectPart().icnIndex );
         }
 
-        for ( const auto & addon : tile.getBottomLayerAddons() ) {
-            if ( addon._objectIcnType == MP2::OBJ_ICN_TYPE_ROAD ) {
-                roadParts.emplace( addon._uid, addon._imageIndex );
-            }
-            else if ( addon._objectIcnType == MP2::OBJ_ICN_TYPE_STREAM ) {
-                streamParts.emplace( addon._uid, addon._imageIndex );
+        for ( const auto & part : tile.getGroundObjectParts() ) {
+            if ( part.icnType == MP2::OBJ_ICN_TYPE_ROAD ) {
+                roadParts.emplace( part._uid, part.icnIndex );
             }
         }
 
@@ -247,7 +407,7 @@ namespace Maps
             if ( object.group == ObjectGroup::ROADS ) {
                 if ( roadParts.empty() ) {
                     // This object was removed from the tile. Delete the object.
-                    info.objects.erase( info.objects.begin() + static_cast<std::vector<Maps::Map_Format::TileObjectInfo>::difference_type>( objectIndex ) );
+                    info.objects.erase( info.objects.begin() + static_cast<std::vector<Map_Format::TileObjectInfo>::difference_type>( objectIndex ) );
                     continue;
                 }
 
@@ -255,17 +415,6 @@ namespace Maps
                 object.index = roadParts.begin()->second;
                 roadParts.erase( roadParts.begin() );
             }
-            else if ( object.group == ObjectGroup::STREAMS ) {
-                if ( streamParts.empty() ) {
-                    // This object was removed from the tile. Delete the object.
-                    info.objects.erase( info.objects.begin() + static_cast<std::vector<Maps::Map_Format::TileObjectInfo>::difference_type>( objectIndex ) );
-                    continue;
-                }
-
-                object.id = streamParts.begin()->first;
-                object.index = streamParts.begin()->second;
-                streamParts.erase( streamParts.begin() );
-            }
 
             ++objectIndex;
         }
@@ -274,10 +423,6 @@ namespace Maps
             addObjectToTile( info, ObjectGroup::ROADS, index, uid );
         }
 
-        for ( const auto & [uid, index] : streamParts ) {
-            addObjectToTile( info, ObjectGroup::STREAMS, index, uid );
-        }
-
         info.terrainIndex = tile.getTerrainImageIndex();
         info.terrainFlag = tile.getTerrainFlags();
     }
@@ -297,7 +442,7 @@ namespace Maps
             auto [heroMetadata, isMetadataEmplaced] = map.heroMetadata.try_emplace( uid );
             assert( isMetadataEmplaced );
 
-            const auto & objects = Maps::getObjectsByGroup( group );
+            const auto & objects = getObjectsByGroup( group );
             assert( index < objects.size() );
             // Set race according the object metadata.
             heroMetadata->second.race = Race::IndexToRace( static_cast<int>( objects[index].metadata[1] ) );
@@ -306,7 +451,7 @@ namespace Maps
             auto [metadata, isMetadataEmplaced] = map.castleMetadata.try_emplace( uid );
             assert( isMetadataEmplaced );
 
-            const auto & objects = Maps::getObjectsByGroup( group );
+            const auto & objects = getObjectsByGroup( group );
             assert( index < objects.size() );
             // Add town or castle main buildings.
             metadata->second.builtBuildings.push_back( objects[index].metadata[1] == 0 ? BUILD_TENT : BUILD_CASTLE );
@@ -326,8 +471,8 @@ namespace Maps
             (void)isMetadataEmplaced;
 #endif
         }
-        else if ( group == Maps::ObjectGroup::ADVENTURE_MISCELLANEOUS ) {
-            const auto & objects = Maps::getObjectsByGroup( group );
+        else if ( group == ObjectGroup::ADVENTURE_MISCELLANEOUS ) {
+            const auto & objects = getObjectsByGroup( group );
 
             assert( index < objects.size() );
             const auto objectType = objects[index].objectType;
@@ -364,8 +509,8 @@ namespace Maps
                 break;
             }
         }
-        else if ( group == Maps::ObjectGroup::ADVENTURE_WATER ) {
-            const auto & objects = Maps::getObjectsByGroup( group );
+        else if ( group == ObjectGroup::ADVENTURE_WATER ) {
+            const auto & objects = getObjectsByGroup( group );
 
             assert( index < objects.size() );
             const auto objectType = objects[index].objectType;
@@ -378,8 +523,8 @@ namespace Maps
 #endif
             }
         }
-        else if ( group == Maps::ObjectGroup::ADVENTURE_ARTIFACTS ) {
-            assert( index < Maps::getObjectsByGroup( group ).size() );
+        else if ( group == ObjectGroup::ADVENTURE_ARTIFACTS ) {
+            assert( index < getObjectsByGroup( group ).size() );
 
             const auto [dummy, isMetadataEmplaced] = map.standardMetadata.try_emplace( uid );
             assert( isMetadataEmplaced );
@@ -390,6 +535,91 @@ namespace Maps
         }
     }
 
+    bool addStream( Map_Format::MapFormat & map, const int32_t tileId )
+    {
+        assert( tileId >= 0 && map.tiles.size() > static_cast<size_t>( tileId ) );
+
+        Map_Format::TileInfo & thisTile = map.tiles[tileId];
+
+        if ( isStreamPresent( thisTile ) || Ground::getGroundByImageIndex( thisTile.terrainIndex ) == Ground::WATER ) {
+            // We cannot place streams on the water or on already placed streams.
+            return false;
+        }
+
+        // Force set stream on this tile and update its sprite.
+        updateStreamObjectOnMapTile( map, tileId, true );
+
+        updateStreamsAround( map, tileId );
+
+        if ( Ground::doesTerrainImageIndexContainEmbeddedObjects( thisTile.terrainIndex ) ) {
+            // We need to set terrain image without extra objects under the stream.
+            thisTile.terrainIndex = Ground::getRandomTerrainImageIndex( Ground::getGroundByImageIndex( thisTile.terrainIndex ), false );
+        }
+
+        return true;
+    }
+
+    void updateStreamsAround( Map_Format::MapFormat & map, const int32_t centerTileId )
+    {
+        // For streams we should update only the next four directions.
+        for ( const int direction : { Direction::LEFT, Direction::TOP, Direction::RIGHT, Direction::BOTTOM } ) {
+            if ( isValidDirection( centerTileId, direction ) ) {
+                updateStreamObjectOnMapTile( map, GetDirectionIndex( centerTileId, direction ), false );
+            }
+        }
+    }
+
+    void updateStreamsToDeltaConnection( Map_Format::MapFormat & map, const int32_t tileId, const int deltaDirection )
+    {
+        if ( !isValidDirection( tileId, deltaDirection ) ) {
+            return;
+        }
+
+        const int32_t nextTileId = GetDirectionIndex( tileId, deltaDirection );
+
+        if ( !isValidDirection( nextTileId, deltaDirection ) ) {
+            return;
+        }
+
+        updateStreamObjectOnMapTile( map, GetDirectionIndex( nextTileId, deltaDirection ), false );
+    }
+
+    int getRiverDeltaDirectionByIndex( const ObjectGroup group, const int32_t objectIndex )
+    {
+        if ( group != ObjectGroup::LANDSCAPE_MISCELLANEOUS ) {
+            return Direction::UNKNOWN;
+        }
+
+        const auto & objectInfo = getObjectInfo( ObjectGroup::LANDSCAPE_MISCELLANEOUS, objectIndex );
+
+        assert( !objectInfo.groundLevelParts.empty() );
+
+        const auto & firstObjectPart = objectInfo.groundLevelParts.front();
+
+        // Yes, the below code is very hacky but so far this is the best we can do.
+        if ( firstObjectPart.icnType == MP2::OBJ_ICN_TYPE_OBJNMUL2 ) {
+            switch ( firstObjectPart.icnIndex ) {
+            case 2U:
+                return Direction::TOP;
+            case 11U:
+                return Direction::BOTTOM;
+            case 218U + 2U:
+                return Direction::LEFT;
+            case 218U + 11U:
+                return Direction::RIGHT;
+            default:
+                break;
+            }
+        }
+
+        return Direction::UNKNOWN;
+    }
+
+    bool isRiverDeltaObject( const ObjectGroup group, const int32_t objectIndex )
+    {
+        return getRiverDeltaDirectionByIndex( group, objectIndex ) != Direction::UNKNOWN;
+    }
+
     bool updateMapPlayers( Map_Format::MapFormat & map )
     {
         static_assert( Color::BLUE == 1 << 0, "The kingdom color values have changed. You are going to break map format!" );
@@ -409,7 +639,7 @@ namespace Maps
         static_assert( Race::MULT == 1 << 6, "The race values have changed. You are going to break map format!" );
         static_assert( Race::RAND == 1 << 7, "The race values have changed. You are going to break map format!" );
 
-        constexpr size_t mainColors{ KINGDOMMAX };
+        constexpr size_t mainColors{ maxNumOfPlayers };
 
         if ( map.playerRace.size() != mainColors ) {
             // Possibly corrupted map.
@@ -538,11 +768,19 @@ namespace Maps
                 for ( auto iter = map.alliances.begin(); iter != map.alliances.end(); ) {
                     uint8_t & allianceColors = *iter;
 
+                    // Only available players should be in the alliances.
+                    allianceColors &= map.availablePlayerColors;
+
                     for ( size_t i = 0; i < mainColors; ++i ) {
                         const uint8_t color = static_cast<uint8_t>( 1 << i );
-                        if ( ( allianceColors & color ) != 0 && usedAllianceColors[i] ) {
-                            // This color is used in another alliance. Remove it from here.
-                            allianceColors = allianceColors & ( ~color );
+                        if ( ( allianceColors & color ) != 0 ) {
+                            if ( usedAllianceColors[i] ) {
+                                // This color is used in another alliance. Remove it from here.
+                                allianceColors = allianceColors & ( ~color );
+                            }
+                            else {
+                                usedAllianceColors[i] = true;
+                            }
                         }
                     }
 
@@ -608,13 +846,13 @@ namespace Maps
                 }
 
                 switch ( objectGroup ) {
-                case Maps::ObjectGroup::KINGDOM_TOWNS: {
+                case ObjectGroup::KINGDOM_TOWNS: {
                     if ( object.index >= townObjects.size() ) {
                         assert( 0 );
                         continue;
                     }
 
-                    const uint32_t color = Color::IndexToColor( Maps::getTownColorIndex( map, tileIndex, object.id ) );
+                    const uint32_t color = Color::IndexToColor( getTownColorIndex( map, tileIndex, object.id ) );
                     if ( color != conditionMetadata[1] ) {
                         // Current town color is incorrect.
                         continue;
@@ -622,7 +860,7 @@ namespace Maps
 
                     return true;
                 }
-                case Maps::ObjectGroup::KINGDOM_HEROES: {
+                case ObjectGroup::KINGDOM_HEROES: {
                     if ( object.index >= heroObjects.size() ) {
                         assert( 0 );
                         continue;
@@ -647,17 +885,17 @@ namespace Maps
         };
 
         switch ( map.victoryConditionType ) {
-        case Maps::FileInfo::VICTORY_CAPTURE_TOWN:
-            if ( !checkSpecialCondition( map.victoryConditionMetadata, Maps::ObjectGroup::KINGDOM_TOWNS ) ) {
+        case FileInfo::VICTORY_CAPTURE_TOWN:
+            if ( !checkSpecialCondition( map.victoryConditionMetadata, ObjectGroup::KINGDOM_TOWNS ) ) {
                 map.victoryConditionMetadata.clear();
-                map.victoryConditionType = Maps::FileInfo::VICTORY_DEFEAT_EVERYONE;
+                map.victoryConditionType = FileInfo::VICTORY_DEFEAT_EVERYONE;
             }
 
             break;
-        case Maps::FileInfo::VICTORY_KILL_HERO:
-            if ( !checkSpecialCondition( map.victoryConditionMetadata, Maps::ObjectGroup::KINGDOM_HEROES ) ) {
+        case FileInfo::VICTORY_KILL_HERO:
+            if ( !checkSpecialCondition( map.victoryConditionMetadata, ObjectGroup::KINGDOM_HEROES ) ) {
                 map.victoryConditionMetadata.clear();
-                map.victoryConditionType = Maps::FileInfo::VICTORY_DEFEAT_EVERYONE;
+                map.victoryConditionType = FileInfo::VICTORY_DEFEAT_EVERYONE;
             }
 
             break;
@@ -666,17 +904,17 @@ namespace Maps
         }
 
         switch ( map.lossConditionType ) {
-        case Maps::FileInfo::LOSS_TOWN:
-            if ( !checkSpecialCondition( map.lossConditionMetadata, Maps::ObjectGroup::KINGDOM_TOWNS ) ) {
+        case FileInfo::LOSS_TOWN:
+            if ( !checkSpecialCondition( map.lossConditionMetadata, ObjectGroup::KINGDOM_TOWNS ) ) {
                 map.lossConditionMetadata.clear();
-                map.lossConditionType = Maps::FileInfo::LOSS_EVERYTHING;
+                map.lossConditionType = FileInfo::LOSS_EVERYTHING;
             }
 
             break;
-        case Maps::FileInfo::LOSS_HERO:
-            if ( !checkSpecialCondition( map.lossConditionMetadata, Maps::ObjectGroup::KINGDOM_HEROES ) ) {
+        case FileInfo::LOSS_HERO:
+            if ( !checkSpecialCondition( map.lossConditionMetadata, ObjectGroup::KINGDOM_HEROES ) ) {
                 map.lossConditionMetadata.clear();
-                map.lossConditionType = Maps::FileInfo::LOSS_EVERYTHING;
+                map.lossConditionType = FileInfo::LOSS_EVERYTHING;
             }
 
             break;
@@ -738,7 +976,7 @@ namespace Maps
 
     bool isJailObject( const ObjectGroup group, const uint32_t index )
     {
-        return ( group == Maps::ObjectGroup::ADVENTURE_MISCELLANEOUS && getObjectInfo( group, static_cast<int32_t>( index ) ).objectType == MP2::OBJ_JAIL );
+        return ( group == ObjectGroup::ADVENTURE_MISCELLANEOUS && getObjectInfo( group, static_cast<int32_t>( index ) ).objectType == MP2::OBJ_JAIL );
     }
 
     uint32_t getBuildingsFromVector( const std::vector<uint32_t> & buildingsVector )
