@@ -31,7 +31,7 @@
 #include <list>
 #include <locale>
 #include <map>
-#include <ostream>
+#include <sstream>
 #include <type_traits>
 #include <utility>
 
@@ -125,7 +125,7 @@ namespace
                 }
             }
 
-            uniqueMaps.try_emplace( System::GetBasename( mapFile ), std::move( fi ) );
+            uniqueMaps.try_emplace( System::GetFileName( mapFile ), std::move( fi ) );
         }
 
         MapsFileInfoList result;
@@ -633,10 +633,23 @@ bool Maps::FileInfo::WinsCompAlsoWins() const
     return compAlsoWins && ( ( GameOver::WINS_TOWN | GameOver::WINS_GOLD ) & ConditionWins() );
 }
 
+std::string Maps::FileInfo::getSummary() const
+{
+    std::ostringstream os;
+
+    os << "Map name: " << name << std::endl
+       << "File name: " << filename << std::endl
+       << "Description: " << description << std::endl
+       << "Size: " << width << "x" << height << std::endl
+       << "Difficulty: " << Difficulty::String( difficulty ) << std::endl;
+
+    return os.str();
+}
+
 OStreamBase & Maps::operator<<( OStreamBase & stream, const FileInfo & fi )
 {
-    // Only the basename of map filename (fi.file) is saved
-    stream << System::GetBasename( fi.filename ) << fi.name << fi.description << fi.width << fi.height << fi.difficulty << static_cast<uint8_t>( maxNumOfPlayers );
+    // Only the filename part of the path to the map file is saved
+    stream << System::GetFileName( fi.filename ) << fi.name << fi.description << fi.width << fi.height << fi.difficulty << static_cast<uint8_t>( maxNumOfPlayers );
 
     static_assert( std::is_same_v<decltype( fi.races ), std::array<uint8_t, maxNumOfPlayers>> );
     static_assert( std::is_same_v<decltype( fi.unions ), std::array<uint8_t, maxNumOfPlayers>> );
@@ -655,7 +668,7 @@ IStreamBase & Maps::operator>>( IStreamBase & stream, FileInfo & fi )
 {
     uint8_t kingdommax = 0;
 
-    // Only the basename of map filename (fi.file) is loaded
+    // Only the filename part of the path to the map file is loaded
     stream >> fi.filename >> fi.name >> fi.description >> fi.width >> fi.height >> fi.difficulty >> kingdommax;
 
     static_assert( std::is_same_v<decltype( fi.races ), std::array<uint8_t, maxNumOfPlayers>> );
