@@ -1,6 +1,6 @@
 /***************************************************************************
  *   fheroes2: https://github.com/ihhub/fheroes2                           *
- *   Copyright (C) 2019 - 2024                                             *
+ *   Copyright (C) 2019 - 2025                                             *
  *                                                                         *
  *   Free Heroes2 Engine: http://sourceforge.net/projects/fheroes2         *
  *   Copyright (C) 2013 by Andrey Afletdinov <fheroes2@gmail.com>          *
@@ -20,8 +20,8 @@
  *   Free Software Foundation, Inc.,                                       *
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
  ***************************************************************************/
-#ifndef H2MAPS_OBJECTS_H
-#define H2MAPS_OBJECTS_H
+
+#pragma once
 
 #include <cstdint>
 #include <initializer_list>
@@ -32,14 +32,15 @@
 #include "artifact.h"
 #include "position.h"
 #include "resource.h"
+#include "skill.h"
 
 class IStreamBase;
 class OStreamBase;
 
-class MapObjectSimple : public MapPosition
+class MapBaseObject : public MapPosition
 {
 public:
-    MapObjectSimple() = default;
+    MapBaseObject() = default;
 
     uint32_t GetUID() const
     {
@@ -58,13 +59,13 @@ public:
     }
 
 protected:
-    friend OStreamBase & operator<<( OStreamBase & stream, const MapObjectSimple & obj );
-    friend IStreamBase & operator>>( IStreamBase & stream, MapObjectSimple & obj );
+    friend OStreamBase & operator<<( OStreamBase & stream, const MapBaseObject & obj );
+    friend IStreamBase & operator>>( IStreamBase & stream, MapBaseObject & obj );
 
     uint32_t uid{ 0 };
 };
 
-struct MapEvent : public MapObjectSimple
+struct MapEvent final : public MapBaseObject
 {
     MapEvent() = default;
 
@@ -84,13 +85,16 @@ struct MapEvent : public MapObjectSimple
 
     Funds resources;
     Artifact artifact;
-    bool computer{ false };
+    bool isComputerPlayerAllowed{ false };
     bool isSingleTimeEvent{ true };
     int colors{ 0 };
     std::string message;
+
+    Skill::Secondary secondarySkill;
+    int32_t experience{ 0 };
 };
 
-struct MapSphinx : public MapObjectSimple
+struct MapSphinx final : public MapBaseObject
 {
     MapSphinx() = default;
 
@@ -136,7 +140,7 @@ struct MapSphinx : public MapObjectSimple
     bool isTruncatedAnswer{ true };
 };
 
-struct MapSign : public MapObjectSimple
+struct MapSign final : public MapBaseObject
 {
     MapSign() = default;
 
@@ -155,5 +159,3 @@ IStreamBase & operator>>( IStreamBase & stream, MapSphinx & obj );
 
 OStreamBase & operator<<( OStreamBase & stream, const MapSign & obj );
 IStreamBase & operator>>( IStreamBase & stream, MapSign & obj );
-
-#endif
