@@ -1,6 +1,6 @@
 /***************************************************************************
  *   fheroes2: https://github.com/ihhub/fheroes2                           *
- *   Copyright (C) 2019 - 2024                                             *
+ *   Copyright (C) 2019 - 2025                                             *
  *                                                                         *
  *   Free Heroes2 Engine: http://sourceforge.net/projects/fheroes2         *
  *   Copyright (C) 2009 by Andrey Afletdinov <fheroes2@gmail.com>          *
@@ -21,8 +21,7 @@
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
  ***************************************************************************/
 
-#ifndef H2MAPSVISIT_H
-#define H2MAPSVISIT_H
+#pragma once
 
 class IndexObject;
 
@@ -39,5 +38,3 @@ namespace Visit
     bool isMonthLife( const IndexObject & visit );
     bool isBattleLife( const IndexObject & visit );
 }
-
-#endif
