@@ -143,21 +143,22 @@ uint32_t Monster::GetShots() const
     return fheroes2::getMonsterData( id ).battleStats.shots;
 }
 
-// Get universal heuristic of Monster type regardless of context; both combat and strategic value
-// Doesn't account for situational special bonuses such as spell immunity
-double Monster::GetMonsterStrength( int attack, int defense ) const
+double Monster::GetMonsterStrength( int attack /* = -1 */, int defense /* = -1 */ ) const
 {
     // TODO: do not use virtual functions when calculating strength for troops without hero's skills.
 
-    // If no modified values were provided then re-calculate
-    // GetAttack and GetDefense will call overloaded versions accounting for Hero bonuses
-    if ( attack == -1 )
+    if ( attack < 0 ) {
+        // This is a virtual function that can be overridden in subclasses and take into account various bonuses (for example, hero bonuses)
         attack = GetAttack();
+    }
 
-    if ( defense == -1 )
+    if ( defense < 0 ) {
+        // This is a virtual function that can be overridden in subclasses and take into account various bonuses (for example, hero bonuses)
         defense = GetDefense();
+    }
 
     const double attackDefense = 1.0 + attack * 0.1 + defense * 0.05;
+
     return attackDefense * fheroes2::getMonsterData( id ).battleStats.monsterBaseStrength;
 }
 
@@ -220,9 +221,12 @@ uint32_t Monster::GetRNDSize() const
 
 bool Monster::isAbilityPresent( const fheroes2::MonsterAbilityType abilityType ) const
 {
-    const std::vector<fheroes2::MonsterAbility> & abilities = fheroes2::getMonsterData( id ).battleStats.abilities;
+    return fheroes2::isAbilityPresent( fheroes2::getMonsterData( id ).battleStats.abilities, abilityType );
+}
 
-    return std::find( abilities.begin(), abilities.end(), fheroes2::MonsterAbility( abilityType ) ) != abilities.end();
+bool Monster::isWeaknessPresent( const fheroes2::MonsterWeaknessType weaknessType ) const
+{
+    return fheroes2::isWeaknessPresent( fheroes2::getMonsterData( id ).battleStats.weaknesses, weaknessType );
 }
 
 Monster Monster::GetDowngrade() const
