@@ -47,10 +47,10 @@ namespace
 int main( int argc, char ** argv )
 {
     if ( argc < 4 ) {
-        const std::string baseName = System::GetBasename( argv[0] );
+        const std::string toolName = System::GetFileName( argv[0] );
 
-        std::cerr << baseName << " extracts sprites in BMP or PNG format (if supported) from the specified TIL file(s) using the specified palette." << std::endl
-                  << "Syntax: " << baseName << " dst_dir palette_file.pal input_file.til ..." << std::endl;
+        std::cerr << toolName << " extracts sprites in BMP or PNG format (if supported) from the specified TIL file(s) using the specified palette." << std::endl
+                  << "Syntax: " << toolName << " dst_dir palette_file.pal input_file.til ..." << std::endl;
         return EXIT_FAILURE;
     }
 
